#ifdef _WIN32
	#include <windows.h>
	#include <shlobj.h>
#endif



#include "worldb.h"
#include "circuit8.cpp"
#include "pill.cpp"
#include "particle.cpp"
#include "../../utils/commands.h"
#include "../../utils/execcmd.h"

void WorldB::CollisionNearOverride(btBroadphasePair& collisionPair, btCollisionDispatcher& dispatcher, const btDispatcherInfo& dispatchInfo)
{
	btRigidBody* b1 = static_cast<btRigidBody*>(collisionPair.m_pProxy0->m_clientObject);
	btRigidBody* b2 = static_cast<btRigidBody*>(collisionPair.m_pProxy1->m_clientObject);
	Entity* e1 = static_cast<Entity*>(b1->getUserPointer());
	Entity* e2 = static_cast<Entity*>(b2->getUserPointer());
	if(e1 == NULL || e2 == NULL)
	{
		//cerr << "Entity cast error in collide callback." << endl;
		return dispatcher.defaultNearCallback(collisionPair, dispatcher, dispatchInfo);
	}
//	else if(e1->type == CRITTER && e2->type == CRITTER)
	else if(e1->type == 0 && e2->type == 0)
	{
		if(e1 == e2)
		{
			//cerr << "Not colliding." << endl;
			return;
		}
	//cerr << "Colliding." << endl;
	}
	dispatcher.defaultNearCallback(collisionPair, dispatcher, dispatchInfo);
}

WorldB::WorldB()
{
	// settings and pointers
	settings = Settings::Instance();
		retinasperrow = settings->getCVarPtr("retinasperrow");
		critter_maxlifetime = settings->getCVarPtr("critter_maxlifetime");
		critter_maxenergy = settings->getCVarPtr("critter_maxenergy");
		critter_autosaveinterval = settings->getCVarPtr("critter_autosaveinterval");
		critter_autoexchangeinterval = settings->getCVarPtr("critter_autoexchangeinterval");
		critter_killhalfat = settings->getCVarPtr("critter_killhalfat");
		critter_retinasize = settings->getCVarPtr("critter_retinasize");
		critter_sightrange = settings->getCVarPtr("critter_sightrange");
		critter_raycastvision = settings->getCVarPtr("critter_raycastvision");
//		maxcritters = settings->getCVarPtr("maxcritters");
//		offspring_queue_length = settings->getCVarPtr("offspring_queue_length");
//		zombie_queue_length = settings->getCVarPtr("zombie_queue_length");


		food_maxlifetime = settings->getCVarPtr("food_maxlifetime");
		food_maxenergy = settings->getCVarPtr("food_maxenergy");
		energy = settings->getCVarPtr("energy");

		pill_maxlifetime = settings->getCVarPtr("pill_maxlifetime");
		pill_energylevel = settings->getCVarPtr("pill_energylevel");
		pill_maxtrip = settings->getCVarPtr("pill_maxtrip");

		condensed_colourmotors = settings->getCVarPtr("condensed_colourmotors");
		bodiless = settings->getCVarPtr("test_bodiless");

//	if (*bodiless) { settings->setCVar("headless", 1); } // too

	statsBuffer = Statsbuffer::Instance();
	critterselection = Critterselection::Instance();

	freeEnergy = *food_maxenergy * *energy;
		
	currentCritterID	= 1;
	insertCritterCounter	= 0;
	autosaveCounter		= 0.0f;
	autoexchangeCounter	= 0.0f;
	insertHight		= 1.0f;

	if (!*bodiless) {
		// vision retina allocation
		items = 4 * 800 * 600;
		retina = (unsigned char*)malloc(items);
		memset(retina, 0, items);
	}

	// home & program directory
	createDirs();

	if (!*bodiless) {
		m_collisionConfiguration = new btDefaultCollisionConfiguration();

		m_dispatcher = new btCollisionDispatcher(m_collisionConfiguration);
		//m_dispatcher = new SpuGatheringCollisionDispatcher(m_collisionConfiguration);


		if(settings->getCVar("body_selfcollisions") == 0)
			m_dispatcher->setNearCallback(&WorldB::CollisionNearOverride);
			
		btVector3 worldAabbMin(-10000,-10000,-10000);
		btVector3 worldAabbMax(10000,10000,10000);
		m_broadphase = new btAxisSweep3 (worldAabbMin, worldAabbMax);
		m_broadphase->getOverlappingPairCache()->setInternalGhostPairCallback(new btGhostPairCallback());
		m_solver = new btSequentialImpulseConstraintSolver;

		m_dynamicsWorld = new btDiscreteDynamicsWorld(m_dispatcher,m_broadphase,m_solver,m_collisionConfiguration);

//	if (*settings->getCVarPtr("shared_screenfireanim")) {
//		circuit8.screendirection = 9;
//	}
	
// 	btVector3 v = m_dynamicsWorld->getGravity();
// 	cerr << v.y() << endl;
// 	m_dynamicsWorld->setGravity( btVector3(0.0f, -50.0f, 0.0f) );

// 	m_dynamicsWorld->getSolverInfo().m_solverMode = SOLVER_USE_WARMSTARTING + SOLVER_SIMD;
	m_dynamicsWorld->getSolverInfo().m_solverMode = SOLVER_SIMD + SOLVER_USE_WARMSTARTING + SOLVER_CACHE_FRIENDLY;
	
	m_dynamicsWorld->getSolverInfo().m_numIterations = 10;
	// raycast
	raycast = new Raycast(m_dynamicsWorld);

	// mousepicker
	mousepicker = new Mousepicker(m_dynamicsWorld);

	// determine vision width
	picwidth = *retinasperrow * (*critter_retinasize+1);
	}


	autocamOn = false; // follow mode
	
	if (!*bodiless) {
		camcritter = 0;
	// reset cam
		resetCamera();
//	oldcameraeuler[0] = -1.5707f;// initial camera
//	oldcameraeuler[1] = 0;       // angle for
//	oldcameraeuler[2] = 0;       // autocam
		oldcamerapos = camera.position;
		oldcameraoff = btVector3(0,8.0f,0);
	}

	pausevotes = 0;
	disrepsevotes = 0;
	dispersevotes = 0;
	firevotes = 0;
	erifvotes = 0;
	rightvotes = 0;
	leftvotes = 0;
	upvotes = 0;
	downvotes = 0;
	mostvotes = 0;
	
	novotesfor = 0;

	scrn_delay_ctr = 0;

	auto_excit_agon_ctr = 0;
	auto_excit_antag_ctr = 0;
	auto_inhib_agon_ctr = 0;
	auto_inhib_antag_ctr = 0;
	auto_potentiator_ctr = 0; 

	particleCtr = 0;
//	shootinterval = 50; // frames (make a cvar)


	circuit8.screenidx = 0;
	if (*settings->getCVarPtr("whitescreen")) {
		for (unsigned int k=0;k<1024;k++) {
			for (unsigned int x=0;x<64;x++) {
				for (unsigned int y=0;y<48;y++) {
					circuit8.moreRed(x,y,1.0f);
					circuit8.moreGreen(x,y,1.0f);
					circuit8.moreBlue(x,y,1.0f);
				}
			}
			circuit8.screenidx++;
		}
	}
	circuit8.screenidx = 128; // arbitrary starting frame, ok
	circuit8.scrndir = scrndir; // for screenshots only

//	if ( settings->getCVar("show_scrn.pnm_every") > 0 )
//	{
		showscrnshotctr = 0;
//	}

	evildeadctr = 0;
	mouseRayHit = false;

//	decrement_autodrug_amts = false;
	
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawWireframe|btIDebugDraw::DBG_DrawConstraints+btIDebugDraw::DBG_DrawConstraintLimits);
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawConstraints+btIDebugDraw::DBG_DrawConstraintLimits);
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawConstraints);
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawConstraintLimits);
// 	m_dynamicsWorld->setDebugDrawer(&debugDrawer);

}

void WorldB::init()
{
	if (!*bodiless) {
	makeFloor();
	}


// 	// reset cam
// 		resetCamera();

	if ( settings->getCVar("autoload") )
		loadAllCritters();

	if ( settings->getCVar("autoload_movie") )
		loadMovieForReal();

	cout << "world initted" << endl;
}

void WorldB::castMouseRay()
{
// 	cerr << "casting" << endl;
	mouseRay = raycast->cast( camera.position.getOrigin(), mouseRayTo );

	mouseRayHit = false;
	if ( mouseRay.hit )
	{
// 		if ( !( mouseRay.hitBody->isStaticObject() || mouseRay.hitBody->isKinematicObject() ) )
// 		{
			Entity* e = static_cast<Entity*>(mouseRay.hitBody->getUserPointer());
			if ( e->type == 1 || e->type == 0 || e->type == 3 || e->type >= 99)
			{
				mouseRayHit = true;
				mouseRayHitEntity = e;
			}
// 		}
	}
}

void WorldB::calcMouseDirection()
{
// 	cerr << "updating mouserayto" << endl;
	mouseRayTo = camera.getScreenDirection(*mousex, *mousey);
}

void WorldB::moveInMouseDirection(bool towards)
{
// 	cerr << "updating mouserayto" << endl;
	if ( towards )
		camera.moveTowards(mouseRayTo.normalized());
	else
		camera.moveAwayFrom(mouseRayTo.normalized());

	calcMouseDirection(); movePickedBodyTo(); // might not be necessary.
						  // just tooltips during
						  // autocam disappear at
						  // certain zooms while
						  // rolling mousewheel
						  // in testworld
}

void WorldB::goto_critter_by_id(unsigned long id) {
	for (unsigned int i=0;i<cRitters.size();i++) {
		if (cRitters[i]->critterID == id) {
			goto_critter(i);
		        stringstream buf;
		        buf << "autocam: following selected critter, " << i << "/" << cRitters.size()-1 << " ID #" << id << " [re-center with backspace, toggle with A]";
			Logbuffer::Instance()->add(buf);
			return;
		}
	}
}

void WorldB::goto_critter(unsigned int i) {
	if (cRitters.size() == 0 || *bodiless || i > cRitters.size() - 1) return;
	// i think the third check above might squash this stupid segfault i had
	// but i've added debug info for goto_critter in case i can crash it that
	// way again
	// it took about 24 hours to reproduce

	cerr << "autocam debug: goto critter #" << i << endl;
	if (!autocamOn) {
		cerr << "autocam was off" << endl;
		autocamOn = true;
		oldcameraoff = btVector3(0,8.0f,0);
		camera.position.getBasis().setEulerZYX( -1.5707f, 0, 0 ); // 1.5707f
		// set a top-down view	
	} else {
		cerr << "autocam was already on" << endl;
		cerr << "get motion state of old critter #" << i << endl;
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[camcritter]->body.bodyparts[0]->body->getMotionState();
		cerr << "get my origin" << endl;
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		cerr << "offset math" << endl;
		oldcameraoff = camera.position.getOrigin()-np;
	}

	camcritter = i;

	cerr << "getting next critter #" << i << "'s motionstate" << endl;
	btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[camcritter]->body.bodyparts[0]->body->getMotionState();
	cerr << "set np" << endl;
	btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();

	cerr << "get myMotionState" << endl;
	myMotionState = (btDefaultMotionState*)cRitters[camcritter]->body.bodyparts[0]->body->getMotionState();
	cerr << "get np" << endl;
	np = myMotionState->m_graphicsWorldTrans.getOrigin();
	cerr << "set oldnp" << endl;
	oldnp = np;
		
	cerr << "set new camera origin" << endl;
	camera.position.setOrigin( (btVector3(np.getX(), np.getY(), np.getZ())+oldcameraoff) ); 
	// go to critter pos
	
	cerr << "and we're fine! yahoo" << endl;
} 

void WorldB::camcritter_up() {
	if (cRitters.size() == 0) return;

	signed int i = camcritter + 1;
	if (i > cRitters.size()-1) i = 0;
	
	goto_critter(i);

        stringstream buf;
        buf << "autocam: critter " << camcritter << " ID #" << cRitters[camcritter]->critterID << " selected (re-center with backspace)";
	Logbuffer::Instance()->add(buf);
}

void WorldB::camcritter_down() {
	if (cRitters.size() == 0) return;

	unsigned int i = camcritter;
	if (i > 0) {i-=1;} else {i=cRitters.size()-1;}
	
	goto_critter(i);

        stringstream buf;
        buf << "autocam: critter " << camcritter << " ID #" << cRitters[camcritter]->critterID << " selected (re-center with backspace)";
	Logbuffer::Instance()->add(buf);
}

void WorldB::autocam() {
	if (autocamOn && !*bodiless) { autocamOn = false; 
		// store camera angle w.r.t. zoomed critter to use when autocam
		// is flipped back on

		//camera.position = oldcamerapos; // but do not reset camera
		                                  // in case user is watching
		                                  // something near zoom still
		//oldcamerapos = camera.position; // to restore angle later
		// camera.position.getBasis().getEulerZYX(oldcameraeuler[0], oldcameraeuler[1], oldcameraeuler[2]);
		// (on next autocam) ^ since this didn't work
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[camcritter]->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		oldcameraoff = camera.position.getOrigin()-np;
	        stringstream buf;
	        buf << "autocam: off";
		Logbuffer::Instance()->add(buf);		
	} else if (cRitters.size()>0 && !*bodiless) {
		autocamOn = true;
		// camcritter = 0;
		
	        stringstream buf;
		buf << "autocam: on, ";
		
		if (camcritter >= cRitters.size()-1) {
			camcritter = cRitters.size() - 1;
			buf << "newest critter";
		} else if (camcritter == 0) {
			buf << "oldest critter";
		} else {
			buf << "critter #" << camcritter << "/" << cRitters.size()-1;
		}
		// oldcamerapos = camera.position;
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[0]->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		oldnp = np;
		
		camera.position.getBasis().setEulerZYX( -1.5707f, -0.0f, -0.0f ); // 1.5707f
		// set an angled top-down view

		//camera.position.getBasis().setEulerZYX( oldcameraeuler[0], oldcameraeuler[1], oldcameraeuler[2] );
		// use last autocam angle (doesnt work)

		//camera.position = oldcamerapos;
		// use last autocam angle (should work)

		oldcameraoff = btVector3(0,8.0f,0);
		camera.position.setOrigin( (btVector3(np.getX(), np.getY(), np.getZ())+oldcameraoff) ); 
		// go to critter pos

		calcMouseDirection(); movePickedBodyTo();

	        buf << " ID#" << cRitters[camcritter]->critterID << " shown (use q/z)";
		Logbuffer::Instance()->add(buf);
	} else {
	        stringstream buf;
	        buf << "autocam: no critters in world";
		Logbuffer::Instance()->add(buf);
	}
//	moveTowardsCritter(0);
}

void WorldB::moveTowardsCritter(unsigned int i) {
		// doesnt work
		// dont use
		// get rid of this
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[i]->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		camera.moveTowards(np);	
}

void WorldB::pickBody(const int& x, const int& y)
{
	if (*bodiless){return;} 
	if ( mouseRayHit )
	{
		if ( mouseRayHitEntity->type == 0 || mouseRayHitEntity->type == 1 || mouseRayHitEntity->type == 3 || mouseRayHitEntity->type >= 99)
		{
			btRigidBody* b = static_cast<btRigidBody*>(mouseRay.hitBody);

			// if critter, and it's the head's ghostobject we touch, overwrite with head's body
			if ( mouseRayHitEntity->type == 0 )
			{
				btCollisionObject* co = static_cast<btCollisionObject*>(mouseRay.hitBody);
				CritterB* c = static_cast<CritterB*>(mouseRayHitEntity);
				if ( co == c->body.mouths[0]->ghostObject )
					b = c->body.mouths[0]->body;
			}
			
			mousepicker->attach( b, mouseRay.hitPosition, camera.position.getOrigin(), mouseRayTo );
			mousepicker->pickedBool = &mouseRayHitEntity->isPicked;
			*mousepicker->pickedBool = true;
		}
	}
}

void WorldB::selectBody(const int& x, const int& y)
{
	if ( mouseRayHit )
		if ( mouseRayHitEntity->type == 0 )
			critterselection->registerCritter(static_cast<CritterB*>(mouseRayHitEntity));
}

void WorldB::movePickedBodyTo()
{
	if ( mousepicker->active )
		mousepicker->moveTo( camera.position.getOrigin(), mouseRayTo );
}

void WorldB::movePickedBodyFrom()
{
	if ( mousepicker->active )
		mousepicker->moveFrom( camera.position.getOrigin() );
}

void WorldB::maybescroll()
{
		return; // this is dumb now anyway
		if (circuit8.upscrolls < 4 && circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { // is it better or worse to do a number of scroll-ups the height is divisible by? going with not for now
			float temprowr[64];
			float temprowg[64];
			float temprowb[64];
			for (int tempidx = 0; tempidx < 1024; tempidx++) {
				for (int tx = 0; tx < 64; tx++) {
					temprowr[tx] = circuit8.screen.screen[tx][1].r[tempidx]; // scrolls up
					temprowg[tx] = circuit8.screen.screen[tx][1].g[tempidx]; // scrolls up
					temprowb[tx] = circuit8.screen.screen[tx][1].b[tempidx]; // scrolls up
				}
				for (int tx = 0; tx < 64; tx++) {
					for (int ty = 0; ty < 48 - 1; ty++) {
						circuit8.screen.screen[tx][ty].r[tempidx] = circuit8.screen.screen[tx][ty+1].r[tempidx]; // scrolls up
						circuit8.screen.screen[tx][ty].g[tempidx] = circuit8.screen.screen[tx][ty+1].g[tempidx]; // scrolls up
						circuit8.screen.screen[tx][ty].b[tempidx] = circuit8.screen.screen[tx][ty+1].b[tempidx]; // scrolls up
					}
				}
				for (int tx = 0; tx < 64; tx++) {
					 circuit8.screen.screen[tx][48 - 1].r[tempidx] = temprowr[tx]; // top row to bottom
					 circuit8.screen.screen[tx][48 - 1].g[tempidx] = temprowg[tx]; // 
					 circuit8.screen.screen[tx][48 - 1].b[tempidx] = temprowb[tx]; // 
				}
			}
			circuit8.upscrolls++;
		} else if (circuit8.upscrolls >= 4 && circuit8.screenidx == 0 && circuit8.framecoloffset == 0) {
			float temprowr[48];
			float temprowg[48];
			float temprowb[48];
			for (int tempidx = 0; tempidx < 1024; tempidx++) {
				for (int tx = 0; tx < 48; tx++) {
					temprowr[tx] = circuit8.screen.screen[1][tx].r[tempidx]; // scrolls right
					temprowg[tx] = circuit8.screen.screen[1][tx].g[tempidx]; // scrolls right
					temprowb[tx] = circuit8.screen.screen[1][tx].b[tempidx]; // scrolls right
				}
				for (int tx = 0; tx < 48; tx++) {
					for (int ty = 0; ty < 64 - 1; ty++) {
						circuit8.screen.screen[ty][tx].r[tempidx] = circuit8.screen.screen[ty+1][tx].r[tempidx]; // scrolls right
						circuit8.screen.screen[ty][tx].g[tempidx] = circuit8.screen.screen[ty+1][tx].g[tempidx]; // scrolls right
						circuit8.screen.screen[ty][tx].b[tempidx] = circuit8.screen.screen[ty+1][tx].b[tempidx]; // scrolls right
					}
				}
				for (int tx = 0; tx < 48; tx++) {
					 circuit8.screen.screen[64 - 1][tx].r[tempidx] = temprowr[tx]; // right col to left
					 circuit8.screen.screen[64 - 1][tx].g[tempidx] = temprowg[tx]; // 
					 circuit8.screen.screen[64 - 1][tx].b[tempidx] = temprowb[tx]; // 
				}
			}
			circuit8.upscrolls = 0;
		}
}

void WorldB::process()
{
//		cout << "worldb::process starts framecoloffset " << circuit8.framecoloffset << endl;

	if (autocamOn && cRitters.size() > 0) {
		//btVector3* cpgo = camera.position.getOrigin();
		//float camY = cpgo.getY()-10.0f;

//		cout << camcritter << "/" << cRitters.size() << endl;
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[camcritter]->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		camera.position.setOrigin(camera.position.getOrigin()+(np-oldnp));
		oldnp = np;		
//		camera_lookdown();
//		camera.follow(myMotionState);
//		camera.position.setOrigin( btVector3(np.getX()+5.5f, np.getY()+8.0f, np.getZ()-0.3f) ); 
//		camera.position.getBasis().setEulerZYX( -1.5707f, -0.1f, -0.5f ); // 1.5707f
		calcMouseDirection(); movePickedBodyTo();
//		cout << "asdf" << endl;
	}

	scrn_delay_ctr++;
	if (scrn_delay_ctr >= settings->getCVar("anim_screen_every") || novotesfor > 0) {
		//scrn_delay_ctr = 0;
	
		// ANIMATE PSYCHIC SUBSTRATE
//		circuit8.screenidx = 0; // non-animated psychic grid or individual grids
//		circuit8.screenidx += 1; if (circuit8.screenidx > 63) circuit8.screenidx = 0; // one-way animated psychic grid


//		maybescroll(); // does this ever happen anymore
		
//		if (circuit8.screenidx > 1021) { circuit8.framecoloffset = 0; }

// bounce ->>	if (circuit8.screenidx < 0)  { if (circuit8.screendirection == 0) {circuit8.screendirection = 1;} else if (circuit8.screendirection == 21) { circuit8.screendirection = 20; } circuit8.screenidx = 0; circuit8.framecoloffset = 0; }
// wrap

//		if ((circuit8.screenidx == 0 && ((circuit8.framecoloffset == 0 && circuit8.screendirection == 21) || circuit8.screendirection == 0) )  { circuit8.screenidx = 1023; if (circuit8.screendirection == 21) { circuit8.framecoloffset = 63; } }
// 		flamoot hi
//		use the line above instead of the one below
//		too lazy to rebuild (have profiling on)
//		note. this still jumps to frame 1 not 0
//		because when it's set to 0 two lines down
//		from this comment it still gets incremented
//		later. these two lines have to be thrown out
//		actually, and let the screendirection check
//		routines below handle flipping at the end, properly
//	        also see the line that check if screenidx == 1023
//		(hidden) it also has to go
//		bye
//		flamoot
		if ((circuit8.screenidx == 0 && (circuit8.framecoloffset == 0 || circuit8.screendirection == 0) && circuit8.screendirection != 1 && circuit8.screendirection != 20) || circuit8.screenidx < 0 )  { circuit8.screenidx = 1022; if (circuit8.screendirection == 21) { circuit8.framecoloffset = 62; } }


// bounce ->>	if (circuit8.screenidx > 1022) { if (circuit8.screendirection == 1) {circuit8.screendirection = 0;} else if (circuit8.screendirection = 20) {circuit8.screendirection = 21; } circuit8.screenidx = 1022; circuit8.framecoloffset = 0; }
// wrap v	
		if ((circuit8.screenidx == 1022 && circuit8.framecoloffset >= 63) || circuit8.screenidx > 1022) { if (circuit8.screendirection != 0 && circuit8.screendirection != 21) { circuit8.screenidx = 0; } if (circuit8.screendirection == 20) { circuit8.framecoloffset = 0; } }


//		maybescroll();
		
		if (circuit8.screendirection == 20) {
			if (circuit8.framecoloffset >= 63) {
				if (circuit8.screenidx >= 1023) { // shouldnt happen
					circuit8.screenidx = 0;
					circuit8.framecoloffset = 0;
				} else { circuit8.framecoloffset = 0; circuit8.screenidx += 1; }
			} else { circuit8.framecoloffset += 1; }
		}

		if (circuit8.screendirection == 21) {
			if (circuit8.framecoloffset == 0) {
				if (circuit8.screenidx == 0) { // shouldnt happen
					circuit8.screenidx = 1022;
					circuit8.framecoloffset = 63;
				} else { circuit8.screenidx -= 1; circuit8.framecoloffset = 63;}
			} else { circuit8.framecoloffset -= 1; }
		}

//		cout << "circuit8.screendirection "<<circuit8.screendirection << endl;

		if (circuit8.screenidx == 1023) { if (circuit8.framecoloffset != 0) { circuit8.screenidx = 0; } } 

		if (circuit8.screendirection == 9 || circuit8.screendirection == 19) { // fire effect
			float temprowr[64];
			float temprowg[64];
			float temprowb[64];
			float differencemultiplier = 0.55f;
//			float fadefactor = 6.01f; // w/ horiz blur
//			float fadefactor = 3.97f; // no horiz blur

			float fadefactor = 3.977f; // no horiz blur
			
			int firedir = 1;
			int tystart = 0;
			int tyend = 48 - 1;
			if (circuit8.screendirection == 19) { firedir = -1; tystart = 48 - 1; tyend = 0;}
			
//			circuit8.framecoloffset = 0;
			for (int ty = tystart; ty != tyend; ty+=firedir) {
				int adjplus = 0;
				if (circuit8.framecoloffset + 1 > 63) {
					adjplus = 1;
				} else {
					adjplus = 0;
				}

//				cout << "fire zero adjplus " << adjplus << " framecoloffset " << circuit8.framecoloffset << endl;

				temprowr[0] = circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx]; 
				temprowg[0] = circuit8.screen.screen[circuit8.framecoloffset][ty].g[circuit8.screenidx]; 
				temprowb[0] = circuit8.screen.screen[circuit8.framecoloffset][ty].b[circuit8.screenidx]; 
//				temprowr[0] += circuit8.screen.screen[circuit8.framecoloffset+1][ty].r[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset+1][ty].r[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx]) * differencemultiplier) / 3.5f; 

// no horiz blur
//				temprowr[0] += circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty].r[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty].r[circuit8.screenidx+adjplus] - circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx+adjplus]) * differencemultiplier) /5.0f;
//				temprowg[0] += circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty].g[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty].g[circuit8.screenidx+adjplus] - circuit8.screen.screen[circuit8.framecoloffset][ty].g[circuit8.screenidx+adjplus]) * differencemultiplier) /5.0f;
//				temprowb[0] += circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty].b[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty].b[circuit8.screenidx+adjplus] - circuit8.screen.screen[circuit8.framecoloffset][ty].b[circuit8.screenidx+adjplus]) * differencemultiplier) /5.0f;

//				temprowr[0] += circuit8.screen.screen[circuit8.framecoloffset-1][ty].r[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset-1][ty].r[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowg[0] += circuit8.screen.screen[circuit8.framecoloffset-1][ty].g[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset-1][ty].g[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].g[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowb[0] += circuit8.screen.screen[circuit8.framecoloffset-1][ty].b[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset-1][ty].b[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].b[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowr[0] += circuit8.screen.screen[circuit8.framecoloffset-1][ty+1].r[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset-1][ty+1].r[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowg[0] += circuit8.screen.screen[circuit8.framecoloffset-1][ty+1].g[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset-1][ty+1].g[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].g[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowb[0] += circuit8.screen.screen[circuit8.framecoloffset-1][ty+1].b[circuit8.screenidx] + ((circuit8.screen.screen[circuit8.framecoloffset-1][ty+1].b[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].b[circuit8.screenidx]) * differencemultiplier) /2.5f;
				temprowr[0] += circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty+firedir].r[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty+firedir].r[circuit8.screenidx+adjplus] - circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx+adjplus]) * differencemultiplier) /2.5f;
				temprowg[0] += circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty+firedir].g[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty+firedir].g[circuit8.screenidx+adjplus] - circuit8.screen.screen[circuit8.framecoloffset][ty].g[circuit8.screenidx+adjplus]) * differencemultiplier) /2.5f;
				temprowb[0] += circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty+firedir].b[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(circuit8.framecoloffset+1)%64][ty+firedir].b[circuit8.screenidx+adjplus] - circuit8.screen.screen[circuit8.framecoloffset][ty].b[circuit8.screenidx+adjplus]) * differencemultiplier) /2.5f;
				temprowr[0] += circuit8.screen.screen[(circuit8.framecoloffset)][ty+firedir].r[circuit8.screenidx] + ((circuit8.screen.screen[(circuit8.framecoloffset)][ty+firedir].r[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].r[circuit8.screenidx]) * differencemultiplier) /2.0f;
				temprowg[0] += circuit8.screen.screen[(circuit8.framecoloffset)][ty+firedir].g[circuit8.screenidx] + ((circuit8.screen.screen[(circuit8.framecoloffset)][ty+firedir].g[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].g[circuit8.screenidx]) * differencemultiplier) /2.0f;
				temprowb[0] += circuit8.screen.screen[(circuit8.framecoloffset)][ty+firedir].b[circuit8.screenidx] + ((circuit8.screen.screen[(circuit8.framecoloffset)][ty+firedir].b[circuit8.screenidx] - circuit8.screen.screen[circuit8.framecoloffset][ty].b[circuit8.screenidx]) * differencemultiplier) /2.0f;
				temprowr[0] = temprowr[0] / fadefactor;
				temprowg[0] = temprowg[0] / fadefactor;
				temprowb[0] = temprowb[0] / fadefactor;			// or four

				for (int tx = 1; tx < 63; tx++) {
					int adjplus;
					int adjminus;
					int adj;
					if (tx+circuit8.framecoloffset + 1 > 63) {
						adjplus = 1;
					} else {
						adjplus = 0;
					}
					if (tx+circuit8.framecoloffset - 1 > 63) {
						adjminus = 1;
					} else {
						adjminus = 0;
					}
					if (tx+circuit8.framecoloffset > 63) {
						adj = 1;
					} else {
						adj = 0;
					}
					
//					cout << "fire one adj "<< adj << " adjminus " << adjminus << " adjplus " << adjplus << endl;


					temprowr[tx] = circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj]; 
					temprowg[tx] = circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj]; 
					temprowb[tx] = circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj]; 

					// classic fire effect only propagates from adjacent rows not 
					// adjacent columns on the same row. try commenting the next
					// six lines if it bugs you, what it does here (propagating
					// sideways)
//					temprowr[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty].r[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty].r[circuit8.screenidx+adjplus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adjplus]) * differencemultiplier) /5.0f;
//					temprowg[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty].g[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty].g[circuit8.screenidx+adjplus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adjplus]) * differencemultiplier) /5.0f;
//					temprowb[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty].b[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty].b[circuit8.screenidx+adjplus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adjplus]) * differencemultiplier) /5.0f;
//					temprowr[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty].r[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty].r[circuit8.screenidx+adjminus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adjminus]) * differencemultiplier) /5.0f;
//					temprowg[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty].g[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty].g[circuit8.screenidx+adjminus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adjminus]) * differencemultiplier) /5.0f;
//					temprowb[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty].b[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty].b[circuit8.screenidx+adjminus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adjminus]) * differencemultiplier) /5.0f;

					temprowr[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty+firedir].r[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty+firedir].r[circuit8.screenidx+adjminus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
					temprowg[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty+firedir].g[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty+firedir].g[circuit8.screenidx+adjminus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
					temprowb[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty+firedir].b[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset-1)%64][ty+firedir].b[circuit8.screenidx+adjminus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
					temprowr[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty+firedir].r[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty+firedir].r[circuit8.screenidx+adjplus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
					temprowg[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty+firedir].g[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty+firedir].g[circuit8.screenidx+adjplus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
					temprowb[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty+firedir].b[circuit8.screenidx+adjplus] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset+1)%64][ty+firedir].b[circuit8.screenidx+adjplus] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
					temprowr[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty+firedir].r[circuit8.screenidx+adj] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty+firedir].r[circuit8.screenidx+adj] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj]) * differencemultiplier) /2.0f;
					temprowg[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty+firedir].g[circuit8.screenidx+adj] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty+firedir].g[circuit8.screenidx+adj] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj]) * differencemultiplier) /2.0f;
					temprowb[tx] += circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty+firedir].b[circuit8.screenidx+adj] + ((circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty+firedir].b[circuit8.screenidx+adj] - circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj]) * differencemultiplier) /2.0f;
					temprowr[tx] = temprowr[tx] / fadefactor;
					temprowg[tx] = temprowg[tx] / fadefactor;
					temprowb[tx] = temprowb[tx] / fadefactor;
				}


				int adjminus;
				int adj;
				if (circuit8.framecoloffset > 1) {
					adjminus = 1;
					adj = 1;
				} else if (circuit8.framecoloffset > 0) {
					adjminus = 0;
					adj = 1;
				} else {
					adjminus = 0;
					adj = 0;
				}

//				cout << "fire two adjminus " << adjminus <<endl;

// no horiz blur
//				temprowr[64 - 1] = circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].r[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].r[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].r[circuit8.screenidx+adjminus]) * differencemultiplier) ;
//				temprowg[64 - 1] = circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].g[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].g[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].g[circuit8.screenidx+adjminus]) * differencemultiplier) ;
//				temprowb[64 - 1] = circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].b[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].b[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].b[circuit8.screenidx+adjminus]) * differencemultiplier) ;

//				temprowr[64 - 1] += circuit8.screen.screen[64 - 1+1][ty].r[circuit8.screenidx] + ((circuit8.screen.screen[64 - 1+1][ty].r[circuit8.screenidx] - circuit8.screen.screen[64 - 1][ty].r[circuit8.screenidx]) * differencemultiplier) /5.0f;
//				temprowg[64 - 1] += circuit8.screen.screen[64 - 1+1][ty].g[circuit8.screenidx] + ((circuit8.screen.screen[64 - 1+1][ty].g[circuit8.screenidx] - circuit8.screen.screen[64 - 1][ty].g[circuit8.screenidx]) * differencemultiplier) /5.0f;
//				temprowb[64 - 1] += circuit8.screen.screen[64 - 1+1][ty].b[circuit8.screenidx] + ((circuit8.screen.screen[64 - 1+1][ty].b[circuit8.screenidx] - circuit8.screen.screen[64 - 1][ty].b[circuit8.screenidx]) * differencemultiplier) /5.0f;

// no horiz blur
//				temprowr[64 - 1] += circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty].r[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty].r[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].r[circuit8.screenidx+adjminus]) * differencemultiplier) /5.0f;
//				temprowg[64 - 1] += circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty].g[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty].g[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].g[circuit8.screenidx+adjminus]) * differencemultiplier) /5.0f;
//				temprowb[64 - 1] += circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty].b[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty].b[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].b[circuit8.screenidx+adjminus]) * differencemultiplier) /5.0f;

				temprowr[64 - 1] = circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty+firedir].r[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty+firedir].r[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].r[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
				temprowg[64 - 1] = circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty+firedir].g[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty+firedir].g[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].g[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
				temprowb[64 - 1] = circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty+firedir].b[circuit8.screenidx+adjminus] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1)-1)%64][ty+firedir].b[circuit8.screenidx+adjminus] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].b[circuit8.screenidx+adj]) * differencemultiplier) /2.5f;
//				temprowr[64 - 1] += circuit8.screen.screen[64 - 1+1][ty+1].r[circuit8.screenidx] + ((circuit8.screen.screen[64 - 1+1][ty+1].r[circuit8.screenidx] - circuit8.screen.screen[64 - 1][ty].r[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowg[64 - 1] += circuit8.screen.screen[64 - 1+1][ty+1].g[circuit8.screenidx] + ((circuit8.screen.screen[64 - 1+1][ty+1].g[circuit8.screenidx] - circuit8.screen.screen[64 - 1][ty].g[circuit8.screenidx]) * differencemultiplier) /2.5f;
//				temprowb[64 - 1] += circuit8.screen.screen[64 - 1+1][ty+1].b[circuit8.screenidx] + ((circuit8.screen.screen[64 - 1+1][ty+1].b[circuit8.screenidx] - circuit8.screen.screen[64 - 1][ty].b[circuit8.screenidx]) * differencemultiplier) /2.5f;
				temprowr[64 - 1] += circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty+firedir].r[circuit8.screenidx+adj] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty+firedir].r[circuit8.screenidx+adj] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].r[circuit8.screenidx+adj]) * differencemultiplier) /2.0f;
				temprowg[64 - 1] += circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty+firedir].g[circuit8.screenidx+adj] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty+firedir].g[circuit8.screenidx+adj] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].g[circuit8.screenidx+adj]) * differencemultiplier) /2.0f;
				temprowb[64 - 1] += circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty+firedir].b[circuit8.screenidx+adj] + ((circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty+firedir].b[circuit8.screenidx+adj] - circuit8.screen.screen[(circuit8.framecoloffset+(64 - 1))%64][ty].b[circuit8.screenidx+adj]) * differencemultiplier) /2.0f;
				temprowr[64 - 1] = temprowr[64 - 1] / fadefactor;
				temprowg[64 - 1] = temprowg[64 - 1] / fadefactor;
				temprowb[64 - 1] = temprowb[64 - 1] / fadefactor;			// or four

				for (int tx = 0; tx < 64; tx++) {
					int adj;
					if (tx+circuit8.framecoloffset > 63) {
						adj = 1;
					} else {
						adj = 0;
					}
//					cout << "fire three adj" << adj <<endl;
					circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj] = abs(temprowr[tx]);
					circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj] = abs(temprowg[tx]);
					circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj] = abs(temprowb[tx]);

					// debug
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj]!=circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj]) { cout << "fire effect nan'd the r channel!" << endl; circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj] = 0; }
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj]!=circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj]) { cout << "fire effect nan'd the g channel!" << endl; circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj] = 0; }
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj]!=circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj]) { cout << "fire effect nan'd the b channel!" << endl; circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj] = 0; }
					// debug
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj] > 1.0f) { circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj] = 1.0f; }
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj] > 1.0f) { circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj] = 1.0f; }
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj] > 1.0f) { circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj] = 1.0f; }
					// debug
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj] < 0) { circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx+adj] = 0; }
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj] < 0) { circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx+adj] = 0; }
					if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj] < 0) { circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx+adj] = 0; }

				}
			}
			for (int tx = 0; tx < 64; tx++) {
				int adj;
				if (tx+circuit8.framecoloffset > 63) {
					adj = 1;
				} else {
					adj = 0;
				}

//				cout << "fire four adj" << adj <<endl;
				circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][tyend].r[circuit8.screenidx+adj] = circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][tyend].r[circuit8.screenidx+adj]/9.0f; // fade bottom row
				circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][tyend].g[circuit8.screenidx+adj] = circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][tyend].g[circuit8.screenidx+adj]/9.0f; //
				circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][tyend].b[circuit8.screenidx+adj] = circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][tyend].b[circuit8.screenidx+adj]/9.0f; //

			}
		}
		
		if (circuit8.screendirection == 30) {
			// disperse
			unsigned int origscreenidx;
			origscreenidx = circuit8.screenidx;
			unsigned int origscreenidxplusadj;
			origscreenidxplusadj = origscreenidx;
			for (int ty = 0; ty < 48; ty++) {
				//break;
				for (int tx = 0; tx < 64; tx++) {
					int adj;
					if (tx+circuit8.framecoloffset > 63) {
						adj = 1;
					} else {
						adj = 0;
					}

					if (origscreenidxplusadj + adj > 1023) { origscreenidxplusadj = 0; } else { origscreenidxplusadj = origscreenidx + adj;}

					
					for (int ttt = 1; ttt < 6; ttt++) {
						for (int tttt = -1; tttt < 2; tttt += 2) {
							int tt;
							float amt;
							amt = 0.00333f/ttt;
							tt = ttt*tttt;
							if (origscreenidx + tt < 0) { circuit8.screenidx = (1024+origscreenidx)-tt; } else if (origscreenidx + tt > 1023) { circuit8.screenidx = (origscreenidx+tt) % 1024; } else { circuit8.screenidx = origscreenidx + tt; }
							if (circuit8.screenidx + adj > 1023) { circuit8.screenidx = 0; } else { circuit8.screenidx += adj; }
													
							if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[origscreenidxplusadj] >= amt) {
								circuit8.moreRed((tx+circuit8.framecoloffset)%64, ty, amt);
								circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[origscreenidxplusadj] -= amt;
							}
							if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[origscreenidxplusadj] >= amt) {
								circuit8.moreGreen((tx+circuit8.framecoloffset)%64, ty, amt);
								circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[origscreenidxplusadj] -= amt;
							}
							if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[origscreenidxplusadj] >= amt) {
								circuit8.moreBlue((tx+circuit8.framecoloffset)%64, ty, amt);
								circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[origscreenidxplusadj] -= amt;
							}
							
						}					
					}
				}
			}
			circuit8.screenidx = origscreenidx;
		}

		if (circuit8.screendirection == 31) {
			// disrepse
			unsigned int origscreenidx;
			origscreenidx = circuit8.screenidx;
			unsigned int origscreenidxplusadj;
			origscreenidxplusadj = origscreenidx;
			for (int ty = 0; ty < 48; ty++) {
				//break;
				for (int tx = 0; tx < 64; tx++) {
					int adj;
					if (tx+circuit8.framecoloffset > 63) {
						adj = 1;
					} else {
						adj = 0;
					}
					if (origscreenidxplusadj + adj > 1023) { origscreenidxplusadj = 0; } else { origscreenidxplusadj = origscreenidx + adj;}

					for (int ttt = 1; ttt < 6; ttt++) {
						for (int tttt = -1; tttt < 2; tttt += 2) {
							int tt;
							float amt;
							amt = 0.00333f/ttt;
							tt = ttt*tttt;
							if (origscreenidx + tt < 0) { circuit8.screenidx = (1024+origscreenidx)-tt; } else if (origscreenidx + tt > 1023) { circuit8.screenidx = (origscreenidx+tt) % 1024; } else { circuit8.screenidx = origscreenidx + tt; }
							if (circuit8.screenidx + adj > 1023) { circuit8.screenidx = 0; } else { circuit8.screenidx += adj; }
													
							if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[origscreenidxplusadj] <= 1 - amt && circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[circuit8.screenidx] >= amt) {
								circuit8.lessRed((tx+circuit8.framecoloffset)%64, ty, amt);
								circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].r[origscreenidxplusadj] += amt;
							}
							if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[origscreenidxplusadj] <= 1 - amt && circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[circuit8.screenidx] >= amt) {
								circuit8.lessGreen((tx+circuit8.framecoloffset)%64, ty, amt);
								circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].g[origscreenidxplusadj] += amt;
							}
							if (circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[origscreenidxplusadj] <= 1 - amt && circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[circuit8.screenidx] >= amt) {
								circuit8.lessBlue((tx+circuit8.framecoloffset)%64, ty, amt);
								circuit8.screen.screen[(tx+circuit8.framecoloffset)%64][ty].b[origscreenidxplusadj] += amt;
							}
							
						}					
					}
				}
			}
			circuit8.screenidx = origscreenidx;
		}

		if (circuit8.screendirection == 1 && circuit8.screenidx < 1023) { circuit8.screenidx += 1;  } else if (circuit8.screendirection == 0 && circuit8.screenidx > 0) { circuit8.screenidx -= 1; }

	// end animate psychic substrate
	}

		if ( settings->getCVar("show_scrn.pnm_every") > 0 )
		{
			if ( showscrnshotctr >= settings->getCVar("show_scrn.pnm_every") ) {
				circuit8.showScreenshot();
				showscrnshotctr = 0;
			} else {
				showscrnshotctr += 1;
			}
		}
		


//		cout << "screenidx updated" << endl;
//	cout << "DFDFF"<<endl;
	// kill half?
		killHalf();

//		cout << "expire" << endl;
	// Remove food
	if (!*bodiless) {
		expireFood();

//		cout << "inserting food" << endl;

	// Autoinsert Food
		autoinsertFood();



//		cout << "done inserting food" << endl;
	}

		autoDrug();
	// remove all dead critters
		expireCritters();
		
//		cout << "expire critters" << endl;

//	cout << "update particles" << endl;



	if (evildeadctr) {
		evildeadctr--;
		newParticles(&eddms, 1.0f, 0, 0, 0.03f, (int)(particledensity*3), 10000, 90000, 750, 175, 100, 10.0f, 0, 0);
		// evil dead
	}
	
//	cout << "done update particles" << endl;


	// Autosave Critters?
		autosaveCritters();

//		cout << "autoexchange" << endl;
	// Autoexchange Critters?
		autoexchangeCritters();

	// Autoinsert Critters?
		autoinsertCritters();

	// do a bullet step
		if (!*bodiless) { m_dynamicsWorld->stepSimulation(Timer::Instance()->bullet_ms / 1000.f); 

//	cout << "render vision" << endl;

		renderVision();
	
//	cout << "render vision done" << endl;	
	
		grabVision();
	}


//	cout << "procewss critters" << endl;
	
	// process all critters
	unsigned int lmax = cRitters.size();

	mostdrugs = (unsigned int) settings->getCVar("pill_maxtrip");
	settings->info_drugtotlev = (unsigned int) lmax * mostdrugs;
	settings->info_drugonelev = 0;
	settings->info_drugtwolev = 0;
	settings->info_drugthrlev = 0;
	settings->info_drugfoulev = 0;
	settings->info_drugfivlev = 0;

	if (lmax > 0) {
		for (unsigned int i=0; i < lmax; i++) {
			CritterB *c = cRitters[i];
			// if (c->brain.tripping[0] < 0) { c->brain.tripping[0] = 0; }
			settings->info_drugonelev += (unsigned int) c->brain.tripping[0];
		}
		for (unsigned int i=0; i < lmax; i++) {
			CritterB *c = cRitters[i];
			// if (c->brain.tripping[1] < 0) { c->brain.tripping[1] = 0; }
			settings->info_drugtwolev += (unsigned int) c->brain.tripping[1];
		}
		for (unsigned int i=0; i < lmax; i++) {
			CritterB *c = cRitters[i];
			// if (c->brain.tripping[2] < 0) { c->brain.tripping[2] = 0; }
			settings->info_drugthrlev += (unsigned int) c->brain.tripping[2];
		}
		for (unsigned int i=0; i < lmax; i++) {
			CritterB *c = cRitters[i];
			// if (c->brain.tripping[3] < 0) { c->brain.tripping[3] = 0; }
			settings->info_drugfoulev += (unsigned int) c->brain.tripping[3];
		}
		for (unsigned int i=0; i < lmax; i++) {
			CritterB *c = cRitters[i];
			// if (c->brain.tripping[4] < 0) { c->brain.tripping[4] = 0; } // or why does that high pink line appear
			settings->info_drugfivlev += (unsigned int) c->brain.tripping[4];
		}
	}

	// set head colors from last frame's blink status. also reset beingBitten
	for( unsigned int i=0; i < lmax; i++)
	{
		CritterB *c = cRitters[i];
//		cout << "lockblinkscreen is " << c->lockblinkscreen << endl;
				
		if (c->blink || c-> lockblink) {
			c->headcolor[1] = 0.1f;
			c->headcolor[0] = 0.3f;
		}

		if (c->blinkscreen || c->lockblinkscreen) {
			c->headcolor[2] = 0.1f;
			if (c->headcolor[0] == 1.0f) { c->headcolor[0] = 0.5f; } else { c->headcolor[0] = 0.2f; }
		}

		if (!c->blink && !c->lockblink && !c->blinkscreen && !c->lockblinkscreen) {
			c->headcolor[1] = 0.0f;
			c->headcolor[2] = 0.0f;
			c->headcolor[0] = 1.0f;
		}
//		c->beingBitten = false; // so touchingcritter never stays on an extra frame
					// but, happens automatically later
	}
	
if (!*bodiless) {
	for( unsigned int i=0; i < lmax; i++) {
			CritterB *c = cRitters[i];
			// TOUCH inputs and references -> find overlappings
			checkCollisions(  c ); // this can set the touchingcritter light
			                       // on other critters now so do the collisions
			                       // in one whole run before processing the
			                       // animals	
	}
}
	for( unsigned int i=0; i < lmax; i++)
	{
//		cout << "make a critter" << endl;
		CritterB *c = cRitters[i];

		if (critterselection->selectedCritter != c) { // does this just compare the pointers' values? or the whole objects
			c->lockblink = false;
			c->lockblinkscreen = false; 
			c->brain.lockdruglevels = false;
		}

		// TOUCH inputs and references -> find overlappings
//			checkCollisions(  c ); // moved outside loop, above

//			cout << "getting group screen" << endl;
//			getGroupScreen(c); // oops i load this in critterb.cpp
					  // search for "overstep"
//			cout << "got group screen" << endl; // oops

//	cout << "process one" << endl;
		// process
			c->process();

		// record critter used energy
			freeEnergy += c->energyUsed;

//	cout << "eat" << endl;
		// process Output Neurons
			if (c->beingBitten) { c->beingBitten=false; c->touchingCritter=false; } // for eat
			if (!*bodiless)
				eat(c); // still ever segfaults?

//	cout << "procreate " << endl;
		// procreation if procreation energy trigger is hit
			procreate(c);

//			cout << "writing to group screen" << endl;
			alterGroupScreen(c);
//			cout << "written to group screen" << endl;

			if (c->votefastdown) {
				downvotes += 1;
			}
			if (c->votefastdown2) {
				downvotes += 2;
			}
			if (c->votefastdown3) {
				downvotes += 3;
			}
			
			if (c->votefastup) {
				upvotes += 1;
			}
			if (c->votefastup2) {
				upvotes += 2;
			}
			if (c->votefastup3) {
				upvotes += 3;
			}
			
			if (c->voteright) {
				rightvotes += 1;
			}
			if (c->voteright2) {
				rightvotes += 2;
			}
			if (c->voteright3) {
				rightvotes += 3;
			}
			
			if (c->voteleft) {
				leftvotes += 1;
			}
			if (c->voteleft2) {
				leftvotes += 2;
			}
			if (c->voteleft3) {
				leftvotes += 3;
			}
			
			if (c->votefire) {
				firevotes += 1;
			}
			if (c->votefire2) {
				firevotes += 2;
			}
			if (c->votefire3) {
				firevotes += 3;
			}
			
			if (c->voteerif) {
				erifvotes += 1;
			}
			if (c->voteerif2) {
				erifvotes += 2;
			}
			if (c->voteerif3) {
				erifvotes += 3;
			}

			if (c->votedisperse) {
				dispersevotes += 1;
			}
			if (c->votedisperse2) {
				dispersevotes += 2;
			}
			if (c->votedisperse3) {
				dispersevotes += 3;
			}

			if (c->votedisrepse) {
				disrepsevotes += 1;
			}
			if (c->votedisrepse2) {
				disrepsevotes += 2;
			}
			if (c->votedisrepse3) {
				disrepsevotes += 3;
			}

			
			if (c->votepause) {
				pausevotes += 1;
			}
			if (c->votepause2) {
				pausevotes += 2;
			}
			if (c->votepause3) {
				pausevotes += 3;
			}
	}

/*	for( unsigned int i=0; i < lmax; i++)
	{
		CritterB *c = cRitters[i];
		if (c->randomshoot && c->randomshootctr == 0) {
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.bodyparts[0]->body->getMotionState();
			newParticles(myMotionState,c->color[0]*((float)randgen->Instance()->get(85,100)/10),c->color[1]*((float)randgen->Instance()->get(85,100)/10),c->color[2]*((float)randgen->Instance()->get(85,100)/10),0,randgen->Instance()->get(particledensity,particledensity*5), 1000, 50000, 60+randgen->Instance()->get(0,60), 40000, 99, 0, 0, 0);
			c->randomshootctr = shootinterval;
		} else if (c->randomshootctr > 0) { c->randomshootctr--; }
	} */
	
	int t = 0;
	int y = 0;
	mostvotes = 0;
	t = upvotes;
	y = 1;
	if (downvotes >= t) { t = downvotes; y = 0; }
	if (leftvotes >= t) { t = leftvotes; y = 21; }
	if (rightvotes >= t) { t = rightvotes; y = 20; }
	if (firevotes >= t) { t = firevotes; y = 9;  }
	if (erifvotes >= t) { t = erifvotes; y = 19;  }
	if (dispersevotes >= t) { t = dispersevotes; y = 30;  }
	if (disrepsevotes >= t) { t = disrepsevotes; y = 31;  }
	if (pausevotes >= t) { t = pausevotes; y = 10; }
	mostvotes = t;

	if (scrn_delay_ctr >= settings->getCVar("anim_screen_every")) {
		scrn_delay_ctr = 0;
		
		if (novotesfor <= 0 && mostvotes > 0) {
			circuit8.screendirection = y;
			if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) {circuit8.framecoloffset = 1;} // is this really necessary for fire, erif, disperse/disrepse and pause? why, if not for the others. i moved it here
		} else if (mostvotes == 0 && novotesfor <= 0) {
//			cout << "setting right scroll" << endl;
			circuit8.screendirection = 20; // default on no votes is right scroll (morphogenic field holds on a frame for 63
						       // ticks, but sensor input changed with framecoloffset)... comment this out if you
						       // want to default to the last-voted-on substrate
		} else {
			novotesfor--; 
		}
	
		upvotes = 0;
		downvotes = 0;
		leftvotes = 0;
		rightvotes = 0;
		firevotes = 0;
		erifvotes = 0;
		dispersevotes = 0;
		disrepsevotes = 0;
		pausevotes = 0;
	
	} else if (novotesfor > 0) {
		novotesfor--;
	} else if (novotesfor <= 0) {
		if (mostvotes > 0) {
			circuit8.screendirection = y;
		}
	}
	
	
	particleCtr += 1;
	if (particleCtr >= particleUpdateFreq) {
		if (!*bodiless) { updateParticles(); } // woohoo
		particleCtr = 0;
	}	
	
//	cout << "end" << endl;

}

void WorldB::getGroupScreen(CritterB* c)
{
//	cout << "getgroupscreen lockblinkscreen is " << c->lockblinkscreen << endl;
	// Read pixels into retina
	if ( !c->blinkscreen && !c->lockblinkscreen && cRitters.size() > 0 )
	{
		for (int i = 0;i<64;i++) { 
			for (int p = 0;p<48;p+=3) {
				if (i + circuit8.framecoloffset < 64) {
//					cout << i << " " << p << endl;
//					cout << circuit8.screen.screen[i][p].g[circuit8.screenidx];
					c->brain.Inputs[100000+i+p*64].output = circuit8.screen.screen[i+circuit8.framecoloffset][p].r[circuit8.screenidx];
					c->brain.Inputs[100000+i+(p*64)+1].output = circuit8.screen.screen[i+circuit8.framecoloffset][p].g[circuit8.screenidx];
					c->brain.Inputs[100000+i+(p*64)+2].output = circuit8.screen.screen[i+circuit8.framecoloffset][p].b[circuit8.screenidx];
//					c->brain.Inputs[100000+(i*600)+p].output = 1;
				} else {
					int adj;
					if (circuit8.screenidx < 1023) {adj = 1;} else {adj = 0;}
					// ^not how to do this. frame 1023 won't
					// display right for them even if 1024
					// appears on tripview they won't see
					// that
					c->brain.Inputs[100000+i+p*64].output = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].r[circuit8.screenidx+adj];
					c->brain.Inputs[100000+i+(p*64)+1].output = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].g[circuit8.screenidx+adj];
					c->brain.Inputs[100000+i+(p*64)+2].output = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].b[circuit8.screenidx+adj];
				}
			}
		}
	} else if (cRitters.size() > 0) { // blinking group screen, write black
		for (int i = 0;i<64;i++) { 
			for (int p = 0;p<48;p+=3) {
				c->brain.Inputs[100000+i+p*64].output = 0; // red
				c->brain.Inputs[100000+i+(p*64)+1].output = 0; // green
				c->brain.Inputs[100000+i+(p*64)+2].output = 0; // blue
			}
		}
	}
}

void WorldB::alterGroupScreen(CritterB* c)
{
//	cout << circuit8.screenidx << " " << c->totalFrames << endl;
//	if (c->totalFrames < 3) { return; } // too noob
	if ( c->writeup ) { if (c->penY > 0) { c->penY -= 1; } else { c->penY = 47; } }
	if ( c->writedown ) { if (c->penY < 47) { c->penY += 1; } else { c->penY = 0; } }
	if ( c->writeleft ) { if (c->penX > 0) { c->penX -= 1; } else { c->penX = 63; } }
	if ( c->writeright ) { if (c->penX < 63) { c->penX += 1;} else {c->penX = 0; } }
	// Read pixels into retina // no
	if ( !c->blinkscreen && !c->lockblinkscreen && cRitters.size() > 0 ) // could allow writes from critters with screen blinked
	{								     // i dont though
		signed int critterXoffset=0;
		signed int critterYoffset=0;
//		if (c->writeleft) { critterXoffset = -1; }
//		if (c->writeright) { critterXoffset += 1; }
//		if (c->writeup) { critterYoffset = -1; }
//		if (c->writedown) { critterYoffset += 1; }
		int intintensity = 1;
		for (unsigned int i = 0;i<3;i++) {
			if (c->writeIntensity[i]) { intintensity = intintensity << 1; } // make the screen 64x64 and use shifts all the time
		}

//		if (c->totalFrames < 250) cout << "new critters write intensity is "<<intintensity<<endl; // get rid of this line

		float intensity;
		intensity = (float) intintensity / 10.0f;

/*		if (evolve_set_to_grey) {
			cout << "altergroupscreen with evolve_set_to_grey" << endl; }
		else
		{ 	cout << "altergroupscreen without evolve_set_to_grey" << endl; }  */

		float oldr;
		float oldg;
		float oldb;
		float newr;
		float newg;
		float newb;
//		float constantwritereward = 0.001f;
		float constantwritereward = 0.0f;
		if (!*condensed_colourmotors) {
			for (int i = 0;i<64;i++) { 
				for (int p = 0;p<48;p++) {
/*					if (c->lessRed[(i*48)+p]) {      circuit8.screen.screen[i][p].r[circuit8.screenidx] -= intensity; if (circuit8.screen.screen[i][p].r[circuit8.screenidx] < 0) circuit8.screen.screen[i][p].r[circuit8.screenidx] = 0; }
					if (c->lessGreen[(i*48)+p]) {    circuit8.screen.screen[i][p].g[circuit8.screenidx] -= intensity; if (circuit8.screen.screen[i][p].g[circuit8.screenidx] < 0) circuit8.screen.screen[i][p].g[circuit8.screenidx] = 0; }
					if (c->lessBlue[(i*48)+p]) {     circuit8.screen.screen[i][p].b[circuit8.screenidx] -= intensity; if (circuit8.screen.screen[i][p].b[circuit8.screenidx] < 0) circuit8.screen.screen[i][p].b[circuit8.screenidx] = 0; }
					if (c->moreRed[(i*48)+p]) {      circuit8.screen.screen[i][p].r[circuit8.screenidx] += intensity; if (circuit8.screen.screen[i][p].r[circuit8.screenidx] > 1.0f) circuit8.screen.screen[i][p].r[circuit8.screenidx] = 1.0f; }
					if (c->moreGreen[(i*48)+p]) {    circuit8.screen.screen[i][p].g[circuit8.screenidx] += intensity; if (circuit8.screen.screen[i][p].g[circuit8.screenidx] > 1.0f) circuit8.screen.screen[i][p].g[circuit8.screenidx] = 1.0f; }
					if (c->moreBlue[(i*48)+p]) {     circuit8.screen.screen[i][p].b[circuit8.screenidx] += intensity; if (circuit8.screen.screen[i][p].b[circuit8.screenidx] > 1.0f) circuit8.screen.screen[i][p].b[circuit8.screenidx] = 1.0f; } */
					unsigned int ti = i;
					unsigned int tp = p;
					if (c->writedown && p > 0) {
						tp -= 1;
					} 
					if (c->writeup && p < 47) {
						tp += 1;
					}
					if (c->writeright && i > 0) {
						ti -= 1;
					}
					if (c->writeleft && i < 63) {
						ti += 1;
					}
					unsigned int motor = ti+tp*64;
					if (i + circuit8.framecoloffset < 64) {
					
						if (evolve_set_to_grey) { // look out appears four times all different
							oldr = circuit8.screen.screen[i+circuit8.framecoloffset][p].r[circuit8.screenidx];
							oldg = circuit8.screen.screen[i+circuit8.framecoloffset][p].g[circuit8.screenidx];
							oldb = circuit8.screen.screen[i+circuit8.framecoloffset][p].b[circuit8.screenidx];
						}
					
						if (c->lessRed[motor]) {      circuit8.lessRed(i+circuit8.framecoloffset,p,intensity); }
						if (c->lessGreen[motor]) {    circuit8.lessGreen(i+circuit8.framecoloffset,p,intensity); }
						if (c->lessBlue[motor]) {     circuit8.lessBlue(i+circuit8.framecoloffset,p,intensity); }
						if (c->moreRed[motor]) {      circuit8.moreRed(i+circuit8.framecoloffset,p,intensity); }
						if (c->moreGreen[motor]) {    circuit8.moreGreen(i+circuit8.framecoloffset,p,intensity); }
						if (c->moreBlue[motor]) {     circuit8.moreBlue(i+circuit8.framecoloffset,p,intensity); }


						if (evolve_set_to_grey) {
							newr = circuit8.screen.screen[i+circuit8.framecoloffset][p].r[circuit8.screenidx];
							newg = circuit8.screen.screen[i+circuit8.framecoloffset][p].g[circuit8.screenidx];
							newb = circuit8.screen.screen[i+circuit8.framecoloffset][p].b[circuit8.screenidx];
							
							float energyreward;
							energyreward = 0;
							
							energyreward += ((abs(( (float) p / 47.0f ) - oldr)-abs((float) p / 47.0f - newr))*3.0f)+constantwritereward; // evolve downward grey gradient. change (float) p / 47.0f to 0.5f to get just grey whole screen
							energyreward += ((abs((float) p / 47.0f - oldg)-abs((float) p / 47.0f - newg))*3.0f)+constantwritereward;
							energyreward += ((abs((float) p / 47.0f - oldb)-abs((float) p / 47.0f - newb))*3.0f)+constantwritereward;
							
//							if (energyreward != energyreward) { cout << "1 energyreward must be nan?! = " << energyreward << endl; energyreward = 0; }
							if (energyreward < -100 or energyreward > 100) { cout << "1 insane energyreward may be nan: " << energyreward << endl; energyreward = 0; }

							c->energyLevel += energyreward;
							freeEnergy -= energyreward;
							if (c->energyLevel > *critter_maxenergy) { 
								freeEnergy += c->energyLevel - *critter_maxenergy;
								c->energyLevel = *critter_maxenergy;
							}
//							cout << "energyreward " << energyreward << endl;
						}
						
					} else {
						int adj;
						if (circuit8.screenidx < 1023) {adj = 1;} else {adj = 0;}
						// ^search "not how to do this"
						circuit8.screenidx += adj;
						
						if (evolve_set_to_grey) {
							oldr = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].r[circuit8.screenidx];
							oldg = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].g[circuit8.screenidx];
							oldb = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].b[circuit8.screenidx];
						}

						if (c->lessRed[motor]) {      circuit8.lessRed(   (i+circuit8.framecoloffset) % 64,p,intensity); }
						if (c->lessGreen[motor]) {    circuit8.lessGreen( (i+circuit8.framecoloffset) % 64,p,intensity); }
						if (c->lessBlue[motor]) {     circuit8.lessBlue(  (i+circuit8.framecoloffset) % 64,p,intensity); }
						if (c->moreRed[motor]) {      circuit8.moreRed(   (i+circuit8.framecoloffset) % 64,p,intensity); }
						if (c->moreGreen[motor]) {    circuit8.moreGreen( (i+circuit8.framecoloffset) % 64,p,intensity); }
						if (c->moreBlue[motor]) {     circuit8.moreBlue(  (i+circuit8.framecoloffset) % 64,p,intensity); }

						if (evolve_set_to_grey) {
							newr = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].r[circuit8.screenidx];
							newg = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].g[circuit8.screenidx];
							newb = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].b[circuit8.screenidx];

							float energyreward;
							energyreward = 0;
							
							energyreward += ((abs((float) p / 47.0f - oldr)-abs((float) p / 47.0f - newr))*3.0f)+constantwritereward;
							energyreward += ((abs((float) p / 47.0f - oldg)-abs((float) p / 47.0f - newg))*3.0f)+constantwritereward;
							energyreward += ((abs((float) p / 47.0f - oldb)-abs((float) p / 47.0f - newb))*3.0f)+constantwritereward;
							
//							if (energyreward != energyreward) { cout << "2 energyreward must be nan?! = " << energyreward << endl; energyreward = 0; }
							if (energyreward < -100 or energyreward > 100) { cout << "2 insane energyreward may be nan: " << energyreward << endl; energyreward = 0; }

							c->energyLevel += energyreward;
							freeEnergy -= energyreward;
							if (c->energyLevel > *critter_maxenergy) { 
								freeEnergy += c->energyLevel - *critter_maxenergy;
								c->energyLevel = *critter_maxenergy;
							}
//							cout << "energyreward " << energyreward << endl;
						}

						
						circuit8.screenidx -= adj;
					}
				}
			}
		} else {
			// condensed colour motors mode
			// now with switches
			// pen only works with condensed for now
			int adj = 0;
			if (circuit8.framecoloffset + c->penX > 63) { adj = 1; } else { adj = 0; }
			circuit8.screenidx += adj;
			if (c->condensedLessRed)   { circuit8.lessRed((c->penX+circuit8.framecoloffset)%64,c->penY,intensity); if (c->LRswitch) { c->LRswitch = false; } else { c->LRswitch = true;} }
			if (c->condensedMoreRed)   { circuit8.moreRed((c->penX+circuit8.framecoloffset)%64,c->penY,intensity); if (c->MRswitch) { c->MRswitch = false; } else { c->MRswitch = true;} }
			if (c->condensedLessGreen) { circuit8.lessGreen((c->penX+circuit8.framecoloffset)%64,c->penY,intensity); if (c->LGswitch) { c->LGswitch = false; } else { c->LGswitch = true;} }
			if (c->condensedMoreGreen) { circuit8.moreGreen((c->penX+circuit8.framecoloffset)%64,c->penY,intensity); if (c->MGswitch) { c->MGswitch = false; } else { c->MGswitch = true;} }
			if (c->condensedLessBlue)  { circuit8.lessBlue((c->penX+circuit8.framecoloffset)%64,c->penY,intensity); if (c->LBswitch) { c->LBswitch = false; } else { c->LBswitch = true;} }
			if (c->condensedMoreBlue)  { circuit8.moreBlue((c->penX+circuit8.framecoloffset)%64,c->penY,intensity); if (c->MBswitch) { c->MBswitch = false; } else { c->MBswitch = true;} }

			// pen writes should have energy rewards then they can explore a failing or booming infovoric fitness landscape but
			// infovorism is too easy anyway it's just a lot of them typify the movie then go extinct

			circuit8.screenidx -= adj; // christ
						  // add vert striping & this
						  // gets worse
			
			for (int i = 0;i<64;i++) { 
				for (int p = 0;p<48;p++) {
					unsigned int tp = p;
					unsigned int ti = i;
					if (c->writeup && p > 0) {
						tp -= 1;
					} 
					if (c->writedown && p < 47) {
						tp += 1;
					}
					if (c->writeleft && i > 0) {
						ti -= 1;
					}
					if (c->writeright && i < 63) {
						ti += 1;
					}
					unsigned int motor = ti+tp*64;
					if (c->condensedColourOp[motor]) {
						if (i + circuit8.framecoloffset < 64) {
							if (evolve_set_to_grey) { // look out appears four times all different
								oldr = circuit8.screen.screen[i+circuit8.framecoloffset][p].r[circuit8.screenidx];
								oldg = circuit8.screen.screen[i+circuit8.framecoloffset][p].g[circuit8.screenidx];
								oldb = circuit8.screen.screen[i+circuit8.framecoloffset][p].b[circuit8.screenidx];
							}


/*
							if (c->condensedLessRed && !c->condensedMoreRed)    {      circuit8.lessRed(i+circuit8.framecoloffset,p,intensity); }
							if (c->condensedLessGreen && !c->condensedMoreGreen)    {    circuit8.lessGreen(i+circuit8.framecoloffset,p,intensity); }
							if (c->condensedLessBlue && !c->condensedMoreBlue)    {     circuit8.lessBlue(i+circuit8.framecoloffset,p,intensity); }
							if (c->condensedMoreRed && !c->condensedLessRed)    {      circuit8.moreRed(i+circuit8.framecoloffset,p,intensity); }
							if (c->condensedMoreGreen && !c->condensedLessGreen)    {    circuit8.moreGreen(i+circuit8.framecoloffset,p,intensity); }
							if (c->condensedMoreBlue && !c->condensedLessBlue)    {     circuit8.moreBlue(i+circuit8.framecoloffset,p,intensity); }
*/

							if (c->LRswitch && !c->MRswitch)  {  circuit8.lessRed(i+circuit8.framecoloffset,p,intensity); }
							if (c->LGswitch && !c->MGswitch)  {  circuit8.lessGreen(i+circuit8.framecoloffset,p,intensity); }
							if (c->LBswitch && !c->MBswitch)  {  circuit8.lessBlue(i+circuit8.framecoloffset,p,intensity); }
							if (c->MRswitch && !c->LRswitch)  {  circuit8.moreRed(i+circuit8.framecoloffset,p,intensity); }
							if (c->MGswitch && !c->LGswitch)  {  circuit8.moreGreen(i+circuit8.framecoloffset,p,intensity); }
							if (c->MBswitch && !c->LBswitch)  {  circuit8.moreBlue(i+circuit8.framecoloffset,p,intensity); }

							if (evolve_set_to_grey) {
								newr = circuit8.screen.screen[i+circuit8.framecoloffset][p].r[circuit8.screenidx];
								newg = circuit8.screen.screen[i+circuit8.framecoloffset][p].g[circuit8.screenidx];
								newb = circuit8.screen.screen[i+circuit8.framecoloffset][p].b[circuit8.screenidx];
							
								float energyreward;
								energyreward = 0;
							
								energyreward += ((abs((float) p / 47.0f - oldr)-abs((float) p / 47.0f - newr))*3.0f)+constantwritereward;
								energyreward += ((abs((float) p / 47.0f - oldg)-abs((float) p / 47.0f - newg))*3.0f)+constantwritereward;
								energyreward += ((abs((float) p / 47.0f - oldb)-abs((float) p / 47.0f - newb))*3.0f)+constantwritereward;

//								if (energyreward != energyreward) { cout << "3 energyreward must be nan?! = " << energyreward << endl; energyreward = 0; }
								if (energyreward < -100 or energyreward > 100) { cout << "3 insane energyreward may be nan: " << energyreward << endl; energyreward = 0; }							

								c->energyLevel += energyreward;
								freeEnergy -= energyreward;
								if (c->energyLevel > *critter_maxenergy) { 
									freeEnergy += c->energyLevel - *critter_maxenergy;
									c->energyLevel = *critter_maxenergy;
								}
//								cout << "energyreward " << energyreward << endl;
							}
						} else {
							if (circuit8.screenidx < 1023) {adj = 1;} else {adj = 0;}
							// ^search "not how to do this"
							circuit8.screenidx += adj;

							if (evolve_set_to_grey) {
								oldr = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].r[circuit8.screenidx];
								oldg = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].g[circuit8.screenidx];
								oldb = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].b[circuit8.screenidx];
							}				
/*
							if (c->condensedLessRed && !c->condensedMoreRed)    {      circuit8.lessRed((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->condensedLessGreen && !c->condensedMoreGreen)    {    circuit8.lessGreen((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->condensedLessBlue && !c->condensedMoreBlue)    {     circuit8.lessBlue((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->condensedMoreRed && !c->condensedLessRed)    {      circuit8.moreRed((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->condensedMoreGreen && !c->condensedLessGreen)    {    circuit8.moreGreen((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->condensedMoreBlue && !c->condensedLessBlue)    {     circuit8.moreBlue((i+circuit8.framecoloffset)%64,p,intensity); }						
*/

							if (c->LRswitch && !c->MRswitch)  {  circuit8.lessRed((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->LGswitch && !c->MGswitch)  {  circuit8.lessGreen((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->LBswitch && !c->MBswitch)  {  circuit8.lessBlue((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->MRswitch && !c->LRswitch)  {  circuit8.moreRed((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->MGswitch && !c->LGswitch)  {  circuit8.moreGreen((i+circuit8.framecoloffset)%64,p,intensity); }
							if (c->MBswitch && !c->LBswitch)  {  circuit8.moreBlue((i+circuit8.framecoloffset)%64,p,intensity); }


							if (evolve_set_to_grey) {
								newr = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].r[circuit8.screenidx];
								newg = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].g[circuit8.screenidx];
								newb = circuit8.screen.screen[(i+circuit8.framecoloffset) % 64][p].b[circuit8.screenidx];
								
								float energyreward;
								energyreward = 0;
							
								energyreward += ((abs((float) p / 47.0f - oldr)-abs((float) p / 47.0f - newr))*3.0f)+constantwritereward;
								energyreward += ((abs((float) p / 47.0f - oldg)-abs((float) p / 47.0f - newg))*3.0f)+constantwritereward;
								energyreward += ((abs((float) p / 47.0f - oldb)-abs((float) p / 47.0f - newb))*3.0f)+constantwritereward;

								if (energyreward != energyreward) { cout << "4 energyreward must be nan?! = " << energyreward << endl; energyreward = 0; }
								if (energyreward < -100 or energyreward > 100) { cout << "4 insane energyreward may be nan: " << energyreward << endl; energyreward = 0; }							

								c->energyLevel += energyreward;
								freeEnergy -= energyreward;
								if (c->energyLevel > *critter_maxenergy) { 
									freeEnergy += c->energyLevel - *critter_maxenergy;
									c->energyLevel = *critter_maxenergy;
								}
//								cout << "energyreward " << energyreward << endl;
							}

							circuit8.screenidx -= adj;
						}
					}
				}
			}
		}
	}
}

void WorldB::killParticle(unsigned int i) {
		if ( Particles[i]->isPicked )
			mousepicker->detach();
		freeEnergy += Particles[i]->energyLevel;
		delete Particles[i];
		Particles.erase(Particles.begin()+i);
}


void WorldB::updateParticles() {
//	return;
	btDefaultMotionState* myMotionState;

	for (unsigned int i = 0; i < Particles.size(); i++) {
		// expire olds
		if ( ++Particles[i]->totalFrames >= Particles[i]->maxLifetime || Particles[i]->killme) {
//			cout << "partikill" << endl;
			killParticle(i);
			i--;
//			cout << "done" << endl;
		} else {
			// position
//			myMotionState = (btDefaultMotionState*)c->body.bodyparts[0]->body->getMotionState();
//			btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
//			np.setX(np.getX()-xdir);
//			np.setY(np.getY()-ydir);
//			np.setZ(np.getZ()-zdir);
//er how to write this back? anyway physics should do it automagically (bullet)

			// color
			for (unsigned int c = 0; c < 4; c++) {
				if (Particles[i]->color[c]) { Particles[i]->color[c] *= (1.06f-(((float)Particles[i]->totalFrames/(float)Particles[i]->maxLifetime))/6.6f); if (Particles[i]->color[c]>1.0f){Particles[i]->color[c]=1.0f;}} // darken with age
			}
		}
	}
}

void WorldB::newParticles(btDefaultMotionState* myMotionState, float r, float g, float b, float a, unsigned int num, signed int minheight, signed int maxheight, unsigned int maxspeed, unsigned int xzspread, unsigned int type, float foodlevel, unsigned int drugtype, float druglevel) {
//		return;
//		cout << "newparticles starts" << endl;
		// btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		
		while (Particles.size() >= maxparticles-num) {

/*			unsigned int s = Particles.size();
			unsigned int oldest = s+1;
			unsigned int hiscore = 0;
			for (unsigned int i = 0; i < s; i++) {
				if (Particles[i]->totalFrames > hiscore) {
					hiscore = Particles[i]->totalFrames;
					oldest = i;
				}
			}
			if (oldest < s+1) {
				delete Particles[oldest];
				Particles.erase(Particles.begin()+oldest);
			} */ // or just:

//			delete Particles[0];
//			Particles.erase(Particles.begin()); // which might be
			killParticle(0);			                                    // the same thing			                                    
		}

		float ox = np.getX();
		float oy = np.getY();
		float oz = np.getZ();
//		if (foodlevel > 0) { freeEnergy -= foodlevel; } // economized
		for (unsigned int i = 0; i < num; i++) {
//			float xd = ((ox-2.5f)+((float)randgen->Instance()->get(0,50))/10.0f);
			float xd = ox+(((float)randgen->Instance()->get(0,xzspread))-xzspread/2.0f);
			float yd = oy+(((float)randgen->Instance()->get(0,(maxheight-minheight)))+minheight);
//			float yd = ((oy+100.0f));
//			float yd = 1000;
//			float zd = ((oz-2.5f)+((float)randgen->Instance()->get(0,50))/10.0f);
//			float zd = oz-50.0f;
			float zd = oz+(((float)randgen->Instance()->get(0,xzspread))-xzspread/2.0f);

			np.setX((ox-0.2f)+((float)randgen->Instance()->get(0,40))/100.0f);
//			np.setY((oy+0.05f)+((float)randgen->Instance()->get(0,30))/100.0f);
			np.setY((oy-0.08f)+((float)randgen->Instance()->get(0,20))/100.0f);
			np.setZ((oz-0.2f)+((float)randgen->Instance()->get(0,40))/100.0f);

			float speed = ((float)randgen->Instance()->get(10,maxspeed*10))/100.0f;
			
			float rx = ((float)randgen->Instance()->get(0,999))/1000.0f;
			float ry = ((float)randgen->Instance()->get(0,999))/1000.0f;
			float rz = ((float)randgen->Instance()->get(0,999))/1000.0f;

//			cout << "ox "<<ox<<" oy "<<oy<<" oz "<<oz<<" xd "<<xd<<" yd "<<yd<<" zd "<<zd<< " speed "<<speed<<endl;

			Particle *npa = new Particle();
		if (foodlevel > 0) { freeEnergy -= foodlevel/(float)num; } // economized
			npa->createBody( m_dynamicsWorld, np, r, g, b, a, xd, yd, zd, speed, rx, ry, rz, type, foodlevel/(float)num, drugtype, druglevel/(float)num);
			Particles.push_back(npa);
		}
//		cout << "newparticles exits" << endl;
}


void WorldB::procreate( CritterB* c )
{
	if ( c->procreate && c->canProcreate )
	{

	if ( settings->getCVar("test_no_offspring") > 0 ) {
			c->energyLevel = c->energyLevel/2.0f;
			c->procreateTimeCount = 0;
			return;
	}

//		cout << "procreating1" << endl;
		if ( cRitters.size() < maxcritters ) {
	//		cout << "procreating2" << endl;

		bool brainmutant = false;
		bool bodymutant = false;
		if ( randgen->Instance()->get(1,100) <= settings->getCVar("brain_mutationrate") )
			brainmutant = true;

		if ( randgen->Instance()->get(1,100) <= settings->getCVar("body_mutationrate") )
			bodymutant = true;

		CritterB *nc;
		if (!*bodiless) {
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.bodyparts[0]->body->getMotionState();
			btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();

//		newParticles(myMotionState, 1.0f, 1.0f, 1.0f, randgen->Instance()->get(10,particledensity*10));
// procreate spray

		// position offset
//		np.setY(insertHight);
			np.setY(np.getY()+1.0f);
// 		if ( np.getX() > settings->getCVar("worldsizeX")/2 )
// 			np.setX(np.getX()-1.0f);
// 		else
// 			np.setX(np.getX()+1.0f);
			nc = new CritterB(*c, currentCritterID++, np, brainmutant, bodymutant, &circuit8);
		} else {
			nc = new CritterB(*c, currentCritterID++, brainmutant, &circuit8);
		}

		// nc->circuit8 = &circuit8;
		//CritterB *nc = new CritterB(*c, currentCritterID++, findPosition(), mutant);

		// display message of birth
			stringstream buf;
			buf << setw(4) << c->critterID << " : " << setw(4) << nc->critterID;
			buf << " ad: " << setw(4) << nc->adamdist;
			buf << " n: " << setw(4) << nc->brain.totalNeurons << " s: " << setw(5) << nc->brain.totalSynapses;

			if ( brainmutant || bodymutant )
			{
				buf << " ";
				if ( brainmutant ) buf << "brain";
				if ( brainmutant && bodymutant ) buf << "+";
				if ( bodymutant ) buf << "body";
				buf << " mutant";
			}

			Textverbosemessage::Instance()->addBirth(buf);
			if (settings->getCVar("headless"))
				cerr << buf.str()<< endl;

		c->procreateTimeCount = 0;

		// split energies in half
			nc->energyLevel = c->energyLevel/2.0f;
			c->energyLevel = nc->energyLevel;

		// reset procreation energy count
			cRitters.push_back( nc );
			nc->calcFramePos(cRitters.size()-1);
			
		} else {
			cout << "not giving an offspring, at max_offspring without finished offspring queue" << endl;
//			cout << "offspring_queue size" << offspring_queue.size() << endl;
//			offspring_queue.push_back( *c );
//			if ( offspring_queue.size() > offspring_queue_length ) {
//				cout << "erase queued offspring" << endl; 
//				offspring_queue.erase(offspring_queue.begin());
//			}
		}
	}
}

void WorldB::eat( CritterB* c )
{
	if ( c->eat )
	{
		if ( c->touchingEdblPrtcl) {
			Particle* p = static_cast<Particle*>(c->touchedEntity);

			float eaten;
			if ( p->energyLevel > 0) {
			eaten = (*critter_maxenergy/1000.0f); // if you look at
							      // the particle
							      // call in eat()
							      // for food this
							      // means it takes
							      // about 5 bites
							      // to eat a food
							      // particle, you
							      // would eat
							      // blood faster
							      // as it holds
							      // less

				if ( c->energyLevel + eaten > *critter_maxenergy )
					eaten -= (c->energyLevel + eaten) - *critter_maxenergy;
				if ( p->energyLevel - eaten <= 0 ) {
					eaten = p->energyLevel;
					p->killme = true;
				}
					
				c->energyLevel += eaten;
				p->energyLevel -= eaten;
			}
			if ( p->druglevel > 0) {
				p->killme=false;
				float eaten = *pill_maxtrip / 1000.0f;
				if ( c->brain.tripping[p->drugtype] + eaten > *pill_maxtrip ) { eaten -= (c->brain.tripping[p->drugtype] + eaten) - *pill_maxtrip; }
				if ( p->druglevel - eaten <= 0) { eaten = p->druglevel; if (p->energyLevel<=0) {p->killme = true;} }
				p->druglevel -= eaten;
				c->brain.tripping[p->drugtype] += eaten;
			}
		} else if ( c->touchingFood )
		{


			//cerr << "eating food" << endl;
			Food* f = static_cast<Food*>(c->touchedEntity);
			
			float eaten = *critter_maxenergy / 100.0f;
			                                                       
			if ( c->energyLevel + eaten > *critter_maxenergy )
				eaten -= (c->energyLevel + eaten) - *critter_maxenergy;
			if ( f->energyLevel - eaten < 0 )
				eaten = f->energyLevel;

			c->energyLevel += eaten;
			f->energyLevel -= eaten;

			btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.mouths[0]->body->getMotionState();
			//btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
			newParticles(myMotionState, 0.0f, 0.9f, 0.0f, 0.27f, randgen->Instance()->get(0,particledensity), -4000, 16000, 60, 1600,100,(*critter_maxenergy/1000.0f),0,0);
			// food spray (FIXME: dont spray if critter has max
			// energy or almost max energy like if eaten is less
			// than max energy / 1000 or something or they can
			// use a piece of food to spray a ton of energy)
			
			if (settings->getCVar("headless")) {
				cerr << "critter " << c->critterID << " ate " << eaten << " remaining food " << food.size() << "/" << *energy << endl;
			}

		}
		else if ( c->touchingPill )
		{
			//cerr << "eating pill" << endl;

			Pill* p = static_cast<Pill*>(c->touchedEntity);
//			p->energyLevel = 10000.0f;

			btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.mouths[0]->body->getMotionState();
			//btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
			newParticles(myMotionState, p->color[0], p->color[1], p->color[2], 0.18f, randgen->Instance()->get(0,particledensity), -4000, 16000, 60, 1600, 101, 0, p->molecule, settings->getCVar("pill_maxtrip")/1000.0f);
			// drug spray

//			float eaten = 100.0f;
			float eaten = *pill_maxtrip / 100.0f;
			if ( c->brain.tripping[p->molecule] + eaten > *pill_maxtrip ) { eaten -= (c->brain.tripping[p->molecule] + eaten) - *pill_maxtrip; }
			if ( p->energyLevel - eaten < 0.0f) {
				eaten = p->energyLevel; }

			c->brain.tripping[p->molecule] += eaten;
			p->energyLevel -= eaten;
			if (p->energyLevel < 0.0f) { p->energyLevel = 0.0f; }
			if (settings->getCVar("headless")) {
				cerr << "critter " << c->critterID << " ate " << eaten << " of pill type " << p->molecule << endl;
			}

		}
		else if ( c->touchingCritter && settings->getCVar("critter_enableomnivores") )
		{
			//cerr << "eating critter" << endl;

			CritterB* ct = static_cast<CritterB*>(c->touchedEntity);
			float eaten = *critter_maxenergy / 1000.0f;

			unsigned int drugtocommute = randgen->Instance()->get(0,4);
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.mouths[0]->body->getMotionState();
			//btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
			newParticles(myMotionState, 0.8f, 0, 0, 0.09f, (unsigned int) ((float) randgen->Instance()->get(0,particledensity) / 2.0f), -4000, 16000, 60, 1600, 102, (*critter_maxenergy/2000.0f), drugtocommute, ct->brain.tripping[drugtocommute]/1000.0f);
			// blood spray^ they have to bite so many times,
			// divide number of particles by 2 for slow
			// machines (so much blood)

			if ( c->energyLevel + eaten > *critter_maxenergy ) {
				eaten -= (c->energyLevel + eaten) - *critter_maxenergy; // hack
			}
			
			if ( ct->energyLevel - eaten < 0 ) { // eaten should always be >= inteaten so no need to check that too
				eaten = ct->energyLevel;
			}

                        int inthalfeaten = (int) (eaten / 2.0f); // to remove and add half

			if (ct->energyLevel >= 2) {
				freeEnergy += (float) inthalfeaten; //  release half of carnivorous energy
        	                //  receive half of carnivorous energy
                	        c->energyLevel += (float) inthalfeaten;
                	        ct->energyLevel -= (float) inthalfeaten*2;
				ct->eaten = true;
			} else {
				c->energyLevel += eaten; // if energy is low
				ct->energyLevel -= eaten;     // give all to eater
				ct->eaten = true;
			}	
			if (settings->getCVar("headless")) {
				cerr << "critter " << c->critterID << " ate " << eaten << " of critter " << ct->critterID << endl;
			}
		}
	}
//	cerr << "done eat" << endl;
}

void WorldB::killHalf()
{
	if ( cRitters.size() >= *critter_killhalfat )
	{
		killHalfOfCritters();
		
		// reduce energy :)
		if ( settings->getCVar("killhalfdecreaseenergybypct") > 0 )
		{
			float dec_amount = ((*food_maxenergy * *energy) - *food_maxenergy) / *food_maxenergy;
			if ( dec_amount >= 0.0f )
			{
				int dec = (dec_amount / 100) * settings->getCVar("killhalfdecreaseenergybypct");
				settings->setCVar("energy", settings->getCVar("energy")-dec );
				//*energy -= dec;
				freeEnergy -= dec * *food_maxenergy;
			}
		}
	}
}

void WorldB::autoinsertCritters()
{
	// insert critter if < minimum
	if ( cRitters.size() < settings->getCVar("mincritters") )
		insertCritter();

	// insert critter if insertcritterevery is reached
	if ( settings->getCVar("insertcritterevery") > 0 )
	{
		if ( insertCritterCounter >= settings->getCVar("insertcritterevery") )
		{
			insertCritter();
			insertCritterCounter = 0;
		}
		else
		{
			insertCritterCounter++;
		}
	}
}

void WorldB::autoDrug() {
	if (cRitters.size() == 0) { return; }

	if (auto_excit_agon_ctr > *settings->getCVarPtr("auto_excit_agon_every")) {
//		cout << "excit_agon_amt" << *settings->getCVarPtr("auto_excit_agon_amt") << endl;
//		cout << "excit_agon_every" << *settings->getCVarPtr("auto_excit_agon_every") << endl;
//		cout << "excit_agon_ctr" << auto_excit_agon_ctr << endl;
		tripall4(settings->getCVar("auto_excit_agon_amt"));
		auto_excit_agon_ctr = 0;
		if (*settings->getCVarPtr("autodecrement_autodrug_amts") && settings->getCVar("auto_excit_agon_amt") > 0) {
			settings->setCVar("auto_excit_agon_amt", settings->getCVar("auto_excit_agon_amt") - settings->getCVar("autodrug_autodecrement_amt"));
		} 
	} else if (*settings->getCVarPtr("auto_excit_agon_every") > 0) {
		auto_excit_agon_ctr++;
	}

	if (auto_inhib_agon_ctr > *settings->getCVarPtr("auto_inhib_agon_every")) {
//		cout << "inhib_agon_every" << settings->getCVarPtr("auto_inhib_agon_every") << endl;
//		cout << "inhib_agon_ctr" << auto_inhib_agon_ctr << endl;
		tripall2(*settings->getCVarPtr("auto_inhib_agon_amt"));
		auto_inhib_agon_ctr = 0;
		if (*settings->getCVarPtr("autodecrement_autodrug_amts") && settings->getCVar("auto_inhib_agon_amt") > 0) {
			settings->setCVar("auto_inhib_agon_amt", settings->getCVar("auto_inhib_agon_amt") - settings->getCVar("autodrug_autodecrement_amt"));
		} 
	} else if (*settings->getCVarPtr("auto_inhib_agon_every") > 0) {
		auto_inhib_agon_ctr++;
	}

	if (auto_excit_antag_ctr > *settings->getCVarPtr("auto_excit_antag_every")) {
		tripall3(*settings->getCVarPtr("auto_excit_antag_amt"));	
		auto_excit_antag_ctr = 0;
		if (*settings->getCVarPtr("autodecrement_autodrug_amts") && settings->getCVar("auto_excit_antag_amt") > 0) {
			settings->setCVar("auto_excit_antag_amt", settings->getCVar("auto_excit_antag_amt") - settings->getCVar("autodrug_autodecrement_amt"));
		} 
	} else if (*settings->getCVarPtr("auto_excit_antag_every") > 0) {
		auto_excit_antag_ctr++;
	}

	if (auto_inhib_antag_ctr > *settings->getCVarPtr("auto_inhib_antag_every")) {
//		cout << "inhib_antag_ctr" << auto_inhib_antag_ctr << endl;
//		cout << "inhib_antag_every" << settings->getCVarPtr("auto_inhib_antag_amt") << endl;
		tripall1(*settings->getCVarPtr("auto_inhib_antag_amt"));	
		auto_inhib_antag_ctr = 0;
		if (*settings->getCVarPtr("autodecrement_autodrug_amts") && settings->getCVar("auto_inhib_antag_amt") > 0) {
			settings->setCVar("auto_inhib_antag_amt", settings->getCVar("auto_inhib_antag_amt") - settings->getCVar("autodrug_autodecrement_amt"));
		} 
	} else if (*settings->getCVarPtr("auto_inhib_antag_every") > 0) {
		auto_inhib_antag_ctr++;
	}

	if (auto_potentiator_ctr > *settings->getCVarPtr("auto_potentiator_every")) {
		tripall5(*settings->getCVarPtr("auto_potentiator_amt"));	
		auto_potentiator_ctr = 0;
		if (*settings->getCVarPtr("autodecrement_autodrug_amts") && settings->getCVar("auto_potentiator_amt") > 0) {
			settings->setCVar("auto_potentiator_amt", settings->getCVar("auto_potentiator_amt") - settings->getCVar("autodrug_autodecrement_amt"));
		} 
	} else if (*settings->getCVarPtr("auto_potentiator_every") > 0) {
		auto_potentiator_ctr++;
	}



	if (settings->getCVar("auto_excit_agon_amt") < 0) {
		settings->setCVar("auto_excit_agon_amt", 0);
	}


	if (settings->getCVar("auto_inhib_agon_amt") < 0) {
		settings->setCVar("auto_inhib_agon_amt", 0);
	}


	if (settings->getCVar("auto_excit_antag_amt") < 0) {
		settings->setCVar("auto_excit_antag_amt", 0);
	}


	if (settings->getCVar("auto_inhib_antag_amt") < 0) {
		settings->setCVar("auto_inhib_antag_amt", 0);
	}


	if (settings->getCVar("auto_potentiator_amt") < 0) {
		settings->setCVar("auto_potentiator_amt", 0);
	}

}

void WorldB::autosaveCritters()
{
	if ( *critter_autosaveinterval > 0 )
	{
		autosaveCounter += ((float)Timer::Instance()->elapsed/1000);
		if ( autosaveCounter > *critter_autosaveinterval )
		{
			autosaveCounter = 0.0f;
			saveAllCritters();
		}
	}
}

void WorldB::autoexchangeCritters()
{
	if ( *critter_autoexchangeinterval > 0 )
	{
		autoexchangeCounter += ((float)Timer::Instance()->elapsed/1000);
		if ( autoexchangeCounter > *critter_autoexchangeinterval )
		{
			autoexchangeCounter = 0.0f;

			// determine exchange directory
			stringstream buf;
			buf << savedir << "/exchange";
			string exchangedir = buf.str();
			
			// save or load? :)
			unsigned int mode = randgen->Instance()->get( 0, 1 );
			
			// always load if critters == 0
			if ( cRitters.size() == 0 || mode )
			{
				vector<string> files;
				dirH.listContentsFull(exchangedir, files);

				// FIXME same routine as load all critters
				for ( unsigned int i = 0; i < files.size(); i++ )
				{
					if ( parseH->Instance()->endMatches( ".cr", files[i] ) )
					{
						stringstream buf;
						buf << "loading " << files[i];
						Logbuffer::Instance()->add(buf);

						string content;
						fileH.open( files[i], content, false ); 
						fileH.rm( files[i] );
						CritterB *c;
						if (!*bodiless) {
							c = new CritterB(content, m_dynamicsWorld, findPosition(), retina, &circuit8);
						} else {
							c = new CritterB(content, &circuit8);
						}
//						c->circuit8 = &circuit8;

						unsigned int error = 0;
						if ( c->retinasize != *critter_retinasize ) error = 1;

						if ( !error)
						{
							cRitters.push_back( c );

							c->critterID = currentCritterID++;
							c->calcFramePos(cRitters.size()-1);

							// start energy
							freeEnergy -= c->energyLevel;
						}
						else
						{
							delete c;
							if ( error == 1 )
							{
								stringstream buf;
								buf << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << files[i];
								Logbuffer::Instance()->add(buf);

								cerr << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << endl;
							}
						}
					}
				}
				stringstream buf;
				buf << "Loaded critters from " << exchangedir;
				Logbuffer::Instance()->add(buf);
				if (settings->getCVar("headless"))
					cerr << buf.str()<< endl;
			}
			else
			{
				// pick one to save
				unsigned int savec = randgen->Instance()->get( 0, cRitters.size()-1 );

				buf << "/" << time(0) << ".cr";
				string filename = buf.str();

				// makde dirs
				if ( !dirH.exists(savedir) )		dirH.make(savedir);
				if ( !dirH.exists(exchangedir) )	dirH.make(exchangedir);

				// save critter
				fileH.save(filename, cRitters[savec]->saveCritterB());

				//cerr << endl << "Saved critters to " << subsavedir << endl << endl;
				stringstream buf2;
				buf2 << "Autoexchange: Saved critter to " << filename;
				Logbuffer::Instance()->add(buf2);
				if (settings->getCVar("headless"))
					cerr << buf2.str()<< endl;
			}
		}
	}
}


void WorldB::autoinsertFood()
{
	if ( freeEnergy >= *food_maxenergy && food.size() < *energy) // added check for food limit (*energy) because
								     // new psychic fitness functions break energy economy badly
								     // while in use. maybe not anymore, though
	{
		insertRandomFood(1, *food_maxenergy);
		freeEnergy -= *food_maxenergy;
		//cerr << "food: " << food.size() << endl;
		if (randgen->Instance()->get(1,3) == 1 && pillsinworld < 128) { WorldB::insertNextPill(1); }
		
	}
}




void WorldB::expireCritters()
{
	for( unsigned int i=0; i < cRitters.size(); i++)
	{
		// see if energy level isn't below 0 -> die, or die of old age
		if ( cRitters[i]->energyLevel <= 0.0f )
		{
			stringstream buf;
			if ( cRitters[i]->eaten )
				buf << setw(4) << cRitters[i]->critterID << " got eaten";
			else
				buf << setw(4) << cRitters[i]->critterID << " starved";
			Textverbosemessage::Instance()->addDeath(buf);
			if (settings->getCVar("headless"))
				cerr << buf.str()<< endl;

			removeCritter(i);
			i--;
		}
		// die of old age
		else if ( cRitters[i]->totalFrames > *critter_maxlifetime )
		{
			stringstream buf;
			buf << setw(4) << cRitters[i]->critterID << " died of age";
			Textverbosemessage::Instance()->addDeath(buf);
			if (settings->getCVar("headless"))
				cerr << buf.str()<< endl;

			removeCritter(i);
			i--;
		}
		// die if y < 100
		else if (!*bodiless)
		{
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[i]->body.bodyparts[0]->body->getMotionState();
			btVector3 pos = myMotionState->m_graphicsWorldTrans.getOrigin();
			
			if ( pos.getY() < clipping_floor )
			{
				stringstream buf;
				buf << setw(4) << cRitters[i]->critterID << " fell in the pit";
				Textverbosemessage::Instance()->addDeath(buf);
				if (settings->getCVar("headless"))
					cerr << buf.str()<< endl;

				evildeadctr = 200; // more is too long
				eddms = *myMotionState;

				removeCritter(i);
				i--;
			}
		}
	}
}

void WorldB::expireFood()
{
	for( unsigned int i=0; i < food.size(); i++)
	{
		// food was eaten
		if ( food[i]->energyLevel <= 0 )
		{
			freeEnergy += food[i]->energyLevel;
			if ( food[i]->isPicked )
				mousepicker->detach();
//			cout << "delete food one" << endl;
			delete food[i];
//			cout << "erase food one" << endl;
			food.erase(food.begin()+i);
			i--;
		}
		// old food, this should remove stuff from corners
		else if ( ++food[i]->totalFrames >= *food_maxlifetime )
		{
			freeEnergy += food[i]->energyLevel; 
			if ( food[i]->isPicked )
				mousepicker->detach();
//			cout << "delete food two" << endl;
			delete food[i]; 
//			cout << "erase food two" << endl;
			food.erase(food.begin()+i);
			i--;
		}
	}
	for (unsigned int i = 0; i < pill.size(); i++) {
		// pill was eaten
		if ( pill[i]->energyLevel <= 0 )
		{
			if (pill[i]->isPicked) { // feed pill
				mousepicker->detach();
			}
//			cout <<"delete pill"<<endl;
			delete pill[i];
//			cout <<"erase pill"<<endl;
			pill.erase(pill.begin()+i); // trying something else:
//			pill[i] = pill[i]->dummy_pill;
//			delete pill[i]->body;
//			pill[i]->removeBody();
			i--;
			pillsinworld--;
		}
		else if ( ++pill[i]->totalFrames >= *pill_maxlifetime )
		{
			//freeEnergy += food[i]->energyLevel; 
			if ( pill[i]->isPicked )
				mousepicker->detach();
//			cout << "delete food two" << endl;
			delete pill[i]; 
//			cout << "erase food two" << endl;
			pill.erase(pill.begin()+i); // pill is an array not a vect
//			pill[i] = pill[i]->dummy_pill;
//			delete pill[i]->body;
//			pill[i]->removeBody();
			i--;
			pillsinworld--;
		}
	}

/*		// die if y < 100
		else
		{
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)food[i]->body.bodyparts[0]->body->getMotionState();
			btVector3 pos = myMotionState->m_graphicsWorldTrans.getOrigin();

			if ( pos.getY() < -100.0f )
			{
				freeEnergy += food[i]->energyLevel;
				if ( food[i]->isPicked )
					mousepicker->detach();
				delete food[i];
				food.erase(food.begin()+i);
				i--;
			}
		}*/
//	}
}

void WorldB::getGeneralStats()
{
	settings->info_totalNeurons = 0;
	settings->info_totalSynapses = 0;
	settings->info_totalAdamDistance = 0;
	settings->info_totalBodyparts = 0;
	settings->info_totalWeight = 0;

	settings->info_critters = cRitters.size();
	settings->info_food = food.size();

	for( unsigned int i=0; i < cRitters.size(); i++)
	{
		settings->info_totalNeurons		+= cRitters[i]->brain.totalNeurons;
		settings->info_totalSynapses		+= cRitters[i]->brain.totalSynapses;
		settings->info_totalAdamDistance	+= cRitters[i]->adamdist;
		settings->info_totalBodyparts		+= cRitters[i]->body.bodyparts.size();
		settings->info_totalWeight		+= cRitters[i]->body.totalWeight;
	}

	statsBuffer->add();

}

void WorldB::checkCollisions( CritterB* c )
{
	// set inputs to false and recheck
		c->touchingFood = false;
		c->touchingPill = false;
		c->touchingEdblPrtcl = false;
		if (c->beingBitten) { c->touchingCritter = true; } else { c->touchingCritter=false; }
		// touchingCritter also tells eat() that the head is touching one which isnt the case
		// when just beingBitten is true. leave it true and reset touchingCritter to false
		// after process() based on beingBitten which we leave true unless this one's head
		// is touching one for real. to the brains touchingCritter means either your head
		// is touching one or one's head is touching you 


	if ( c->body.mouths.size() > 0 )
	{
		btBroadphasePairArray& pairArray = c->body.mouths[0]->ghostObject->getOverlappingPairCache()->getOverlappingPairArray();
		int numPairs = pairArray.size();

		for ( int i=0; i < numPairs; i++ )
		{
			manifoldArray.clear();

			const btBroadphasePair& pair = pairArray[i];

			//unless we manually perform collision detection on this pair, the contacts are in the dynamics world paircache:
			btBroadphasePair* collisionPair = m_dynamicsWorld->getPairCache()->findPair(pair.m_pProxy0,pair.m_pProxy1);
			if (!collisionPair)
				continue;

			if (collisionPair->m_algorithm)
				collisionPair->m_algorithm->getAllContactManifolds(manifoldArray);

			for ( int j = 0; j < manifoldArray.size(); j++ )
			{
				btPersistentManifold* manifold = manifoldArray[j];
				
				btCollisionObject* object1 = static_cast<btCollisionObject*>(manifold->getBody0());
				btCollisionObject* object2 = static_cast<btCollisionObject*>(manifold->getBody1());

				if ( object1->getUserPointer() == c && object2->getUserPointer() == c )
					continue;

				for ( int p = 0; p < manifold->getNumContacts(); p++ )
				{
					const btManifoldPoint &pt = manifold->getContactPoint(p);
					if ( pt.getDistance() < 0.0f )
					{
						void* Collidingobject;
						if ( object1->getUserPointer() != c && object1->getUserPointer() != 0 )
							Collidingobject = object1->getUserPointer();
						else if ( object2->getUserPointer() != c && object2->getUserPointer() != 0 )
							Collidingobject = object2->getUserPointer();
						else 
							continue;


						Entity* e = static_cast<Entity*>(Collidingobject);

						// Touching Edible Particle
						if ( e->type > 99 )
						{
// 							cerr << "touches edible particle" << endl;
							c->touchingEdblPrtcl = true; // this sets no brain input its not important enough
							c->touchedEntity = e;
							if ( e->type == 102 ) {
								c->touchingCritter = true;
							} else if ( e->type == 100) {
								c->touchingFood = true;
							} else if ( e->type == 101) {
								c->touchingPill = true;
							}
							return;
						}
						// Touching Food
						else if ( e->type == 1 )
						{
// 							cerr << "touches food" << endl;
							c->touchingFood = true;
							c->touchedEntity = e;
							return;
						}
						// Touching Critter
						else if ( e->type == 0)
						{
// 							cerr << "touches critter" << endl;
							c->beingBitten = false;
							c->touchingCritter = true;
							c->touchedEntity = e;
							CritterB* tc = static_cast<CritterB*>(Collidingobject);
							tc->beingBitten = true;     // flamoot code
							tc->touchingCritter = true; // beingBitten is just
							//tc->touchedEntity = c; // (no cast)
							                            // to avoid resetting
							return;                     // touchingCritter
						}
						// Touching Pill
						else if ( e->type == 3)
						{
							c->touchingPill = true;
							c->touchedEntity = e;
							return;
						}
/*						// Touching Food
						Food* f = static_cast<Food*>(Collidingobject);
						if ( f )
						{
							if ( f->type == 1 )
							{
								stop = true;
								c->touchingFood = true;
								c->touchedFoodID = f;
							}
							else
							{
								// Touching Critter
								CritterB* b = static_cast<CritterB*>(Collidingobject);
								if ( b->type == 0 )
								{
									stop = true;
									c->touchingCritter = true;
									c->touchedCritterID = b;
								}
							}
						}*/
					}
				}
			}
		}
	}
}


void WorldB::insertCritter()
{
	CritterB *c;
	if (!*bodiless) {
		c = new CritterB(m_dynamicsWorld, currentCritterID++, findPosition(), retina, &circuit8);
		c->calcFramePos(cRitters.size()-1);
	} else {
		c = new CritterB(currentCritterID++, &circuit8);
	}
//	c->circuit8 = &circuit8;	
	cRitters.push_back( c );

	// start energy
	freeEnergy -= c->energyLevel;
}

btVector3 WorldB::findPosition()
{
	if (*bodiless) { return btVector3(0,0,0); } 
	return btVector3( (float)randgen->Instance()->get( 0, 100*settings->getCVar("worldsizeX") ) / 100, insertHight, (float)randgen->Instance()->get( 0, 100*settings->getCVar("worldsizeY") ) / 100 );
}

void WorldB::removeCritter(unsigned int cid)
{
	freeEnergy += cRitters[cid]->energyLevel;

	if ( cRitters[cid]->isPicked )
		mousepicker->detach();
	
	critterselection->unregisterCritterID(cRitters[cid]->critterID);
	critterselection->deselectCritter(cRitters[cid]->critterID);

/*	signed int t = -1; */
	if (autocamOn && cRitters.size() == 1) {
		autocam();
	}
	
/*	if (autocamOn && cid == camcritter && cRitters.size()>1) { // for autocam
						      // so we don't use
						      // the difference between
						      // the killed critter's
						      // position and that of
						      // the critter now at
						      // its id, ever
		t = camcritter;
		if (t+1 < cRitters.size()) {t += 1;} else {t = 0;}
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[t]->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		oldnp = np;
	}  */

	unsigned int newcamcritter = camcritter;
	if (autocamOn) {
		if (cid < camcritter) {
			//goto_critter(camcritter);
			newcamcritter = camcritter-1;
		} else if (cid == camcritter) {
			if (cRitters.size()==cid+1) {
				goto_critter(1); newcamcritter = 0;
			} else {
				goto_critter(cid+1); newcamcritter = cid;
			} // killed critter we were watching
		}
	}
	
	delete cRitters[cid];
	cRitters.erase(cRitters.begin()+cid);

	camcritter = newcamcritter;

	// update higher retina frame positions
	for ( unsigned int c = cid; c < cRitters.size(); c++ )
		cRitters[c]->calcFramePos(c);
}

void WorldB::killHalfOfCritters()
{
	if (!just_kill_oldest) {
		for ( unsigned int c = 0; c < cRitters.size(); c++ )
			removeCritter(c);
	} else {
		if (cRitters.size() > 0) {
			removeCritter(0); // just kill oldest hack
		}
	}
}

void WorldB::renderVision()
{
//	cout << "rendervision sizeof cRitters: "<< cRitters.size() << endl;
	// render critter vision
	if ( !*critter_raycastvision )
		for( unsigned int i=0; i < cRitters.size(); i++)
			if ( cRitters[i]->body.mouths.size() > 0 )
			{
				cRitters[i]->place();
//				cout << "aaaaaaa" << endl;
				drawWithinCritterSight(cRitters[i]);
			}
}

void WorldB::grabVision()
{
	// Read pixels into retina
	if ( !*critter_raycastvision && cRitters.size() > 0 )
	{
		// determine height
		unsigned int picheight = *critter_retinasize;
		unsigned int rows = cRitters.size();
		while ( rows > *retinasperrow )
		{
			picheight += *critter_retinasize;
			rows -= *retinasperrow;
		}
		glReadBuffer(GL_BACK);
		glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
		glReadPixels(0, 0, picwidth, picheight, GL_RGBA, GL_UNSIGNED_BYTE, retina);
	}
}


void WorldB::drawWithoutFaces()
{
//	cout << "wtf" << endl;
 	for( unsigned int i=0; i < cRitters.size(); i++)
		cRitters[i]->draw(false);

//	cout << "WTF" << endl;
	for( unsigned int i=0; i < pill.size(); i++)
		pill[i]->draw();

	for( unsigned int i=0; i < food.size(); i++)
		food[i]->draw();

	for( unsigned int i=0; i < walls.size(); i++)
		walls[i]->draw();

	for( unsigned int i=0; i < Particles.size(); i++)
		Particles[i]->draw();

	// draw floor
// 	grid.draw();
}


void WorldB::drawWithGrid()
{
//	cout << "tfw" << endl;
	for( unsigned int i=0; i < cRitters.size(); i++)
		cRitters[i]->draw(true);

	for( unsigned int i=0; i < food.size(); i++)
		food[i]->draw();

//	printf("%i\n",pill.size());
	for( unsigned int i=0; i < pill.size(); i++)
		pill[i]->draw();

//	printf("aaaaaaa\n");
	for( unsigned int i=0; i < walls.size(); i++)
		walls[i]->draw();

	for( unsigned int i=0; i < Particles.size(); i++)
		Particles[i]->draw();


//	printf("aaaaaaa\n");
// 	m_dynamicsWorld->debugDrawWorld();

//	printf("sdfsdf\n");

	// draw floor
// 	grid.draw();
}


void WorldB::drawWithinCritterSight(CritterB *c)
{
// 	CritterB *c = critters[cid];

//	cout << "drawwithincrittersight" << endl;

	if (!c->blink && !c->lockblink) {
		if ( c->body.mouths.size() > 0 )
		{
			btDefaultMotionState* cmyMotionState = (btDefaultMotionState*)c->body.mouths[0]->body->getMotionState();
			btVector3 cposi = cmyMotionState->m_graphicsWorldTrans.getOrigin();

			// draw everything in it's sight
			float sightrange = (float)*critter_sightrange/10;

			btScalar position[16];

			//		cout << "asdf" << endl;

			for( unsigned int i=0; i < walls.size(); i++)
				walls[i]->draw();

			for( unsigned int i=0; i < food.size(); i++)
			{
				Food *f = food[i];
				if ( cposi.distance( f->myMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					f->draw();
			}

			for( unsigned int i=0; i < pill.size(); i++)
			{
				Pill *p = pill[i];
				if ( cposi.distance( p->myMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					p->draw();
			}

			for( unsigned int i=0; i < Particles.size(); i++)
			{
				Particle *p = Particles[i];
				if ( cposi.distance( p->myMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					p->draw();
			}


			for( unsigned int j=0; j < cRitters.size(); j++)
			{
				CritterB *f = cRitters[j];
				for( unsigned int b=0; b < f->body.bodyparts.size(); b++)
				{
					const btDefaultMotionState* fmyMotionState = static_cast<const btDefaultMotionState*>(f->body.bodyparts[b]->body->getMotionState());
					if ( cposi.distance( fmyMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					{
						fmyMotionState->m_graphicsWorldTrans.getOpenGLMatrix(position);
						glPushMatrix(); 
						glMultMatrixf(position);
						
								glColor4f( f->color[0], f->color[1], f->color[2], 0.0f );

								btVector3 halfExtent = static_cast<const btBoxShape*>(f->body.bodyparts[b]->shape)->getHalfExtentsWithMargin();
								glScaled(halfExtent[0], halfExtent[1], halfExtent[2]);
								
								Displaylists::Instance()->call(0);

						glPopMatrix();
					}
				}
				if ( c->critterID != cRitters[j]->critterID )
				{
					for( unsigned int j=0; j < f->body.mouths.size(); j++)
					{
						if ( cposi.distance( f->body.mouths[j]->ghostObject->getWorldTransform().getOrigin() ) < sightrange )
						{
							f->body.mouths[j]->ghostObject->getWorldTransform().getOpenGLMatrix(position);
						
							glPushMatrix(); 
							glMultMatrixf(position);

									glColor4f( f->headcolor[0], f->headcolor[1], f->headcolor[2], f->headcolor[3] );

									btVector3 halfExtent = static_cast<const btBoxShape*>(f->body.mouths[j]->shape)->getHalfExtentsWithMargin();
									glScaled(halfExtent[0], halfExtent[1], halfExtent[2]);

									Displaylists::Instance()->call(1);
								
							glPopMatrix();
						}
					}
				}
			}
		}
	} //else { cout << "blinked" << endl; }
}

void WorldB::loadAllCritters()
{
	vector<string> files;
	dirH.listContentsFull(loaddir, files);

	for ( unsigned int i = 0; i < files.size(); i++ )
	{
		if ( parseH->Instance()->endMatches( ".cr", files[i] ) )
		{
			stringstream buf;
			buf << "loading " << files[i];
			Logbuffer::Instance()->add(buf);

			string content;
			fileH.open( files[i], content, false ); 

//			cout << "loading " << files[i] << endl;
			CritterB *c = new CritterB(content, m_dynamicsWorld, findPosition(), retina, &circuit8);
//			c->circuit8 = &circuit8;
//			cout << "loaded" << endl;

			unsigned int error = 0;
			if ( c->retinasize != *critter_retinasize ) error = 1;

			if ( !error)
			{
				cRitters.push_back( c );

				c->critterID = currentCritterID++;
				c->calcFramePos(cRitters.size()-1);

				// start energy
				freeEnergy -= c->energyLevel;
				// c->brain.tripping = 0.0f; // should all start 0
			}
			else
			{
				delete c;
				if ( error == 1 )
				{
					stringstream buf;
					buf << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << files[i];
					Logbuffer::Instance()->add(buf);

					cerr << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << endl;
				}
			}
		}
	}
	stringstream buf;
	buf << "Loaded critters from " << loaddir;
	Logbuffer::Instance()->add(buf);
	//cerr << endl << "Loaded critters from " << loaddir << endl << endl;
}


//void WorldB::loadMovie(bool frompanel) {
void WorldB::loadMovie() {
        cmdsettings           command;
        Commands*             cmd;
        Execcmd               execcmd;

	cmd = Commands::Instance();

	// holy christ

	command = execcmd.gen( "gui_togglepanel", "loadpanel" );

//        if (frompanel) {
        	cmd->execCmd( command );
//        }

	loadMovieForReal(); // because of no bool frompanel ... see commands.cpp, execcmd.cpp
}

void WorldB::loadMovieForReal() {
	string moviedir = loaddir;
	moviedir.append("/movie");
	
	stringstream buf2;
	if ( !dirH.exists(moviedir) ) {
		buf2 << moviedir << " -- no such directory";
		Logbuffer::Instance()->add(buf2);
	} else {
		unsigned int aaa = circuit8.load();
		if (aaa == 0) {
			buf2 << "p-screen overwritten with contents of " << moviedir;
		} else if (aaa == 1) {
			buf2 << "p-screen not overwritten, no files found in " << moviedir;
		} else if (aaa == 2) {
			buf2 << "p-screen partially overwritten, some frames not found in " << moviedir;
		}
			
		Logbuffer::Instance()->add(buf2);
	}
}

void WorldB::saveAllCritters()
{
	// determine save directory
	stringstream buf;
	buf << savedir << "/" << settings->profileName;
	string subprofiledir = buf.str();

	circuit8.savedir = subprofiledir;	circuit8.savedir.append("/movie");

	buf << "/" << time(0);
	string subsavedir = buf.str();

	// makde dirs
	if ( !dirH.exists(savedir) )		dirH.make(savedir);
	if ( !dirH.exists(subprofiledir) )	dirH.make(subprofiledir);
	if ( !dirH.exists(subsavedir) )		dirH.make(subsavedir);
	if ( !dirH.exists(circuit8.savedir) )	dirH.make(circuit8.savedir);
	
	circuit8.save();

	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		// determine filename
		stringstream filename;
		filename << subsavedir << "/" << "critter" << i << ".cr";
	
		// save critters
		fileH.save(filename.str(), cRitters[i]->saveCritterB());
	}
 	//cerr << endl << "Saved critters to " << subsavedir << endl << endl;
	stringstream buf2;
	buf2 << "Saved critters to " << subsavedir;
	Logbuffer::Instance()->add(buf2);
	stringstream buf3;
	buf3 << "Saved psychic movie to " << circuit8.savedir;
	Logbuffer::Instance()->add(buf3);

}

void WorldB::clearnovote() {
	novotesfor = 0;
}

void WorldB::tripfirst() {
//	critters[0]->brain.tripping[0] += 100.0;
//	critters[0]->brain.tripping[1] += 100.0;
//	critters[0]->brain.tripping[2] += 100.0;
//	critters[0]->brain.tripping[3] += 100.0;

//	cRitters[0]->brain.tripping[randgen->Instance()->get(0,4)] += 100.0;

//	if      (circuit8.screendirection == 0) { circuit8.screendirection = 2; }
//	else if (circuit8.screendirection == 1) { circuit8.screendirection = 3; }
//	else if (circuit8.screendirection == 2) { circuit8.screendirection = 0; }
//	else if (circuit8.screendirection == 3) { circuit8.screendirection = 1; }
//	else if (circuit8.screendirection == 9) { circuit8.screendirection = 10; }
//	else if (circuit8.screendirection == 10) { circuit8.screendirection = 9; }

	cyclescreenmodes(true);
}

void WorldB::tripfirst2() {
	cyclescreenmodes(false);
}

void WorldB::cyclescreenmodes(bool dir) {
	novotesfor = 100000;
	if	(circuit8.screendirection == 0)  {  if (dir) { circuit8.screendirection = 1; } else { circuit8.screendirection = 2; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } } 
	else if	(circuit8.screendirection == 1)  {  if (dir) { circuit8.screendirection = 10; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 0; } }
	else if	(circuit8.screendirection == 10) {  if (dir) { circuit8.screendirection = 20; } else { circuit8.screendirection = 1; } }
	else if	(circuit8.screendirection == 20) {  if (dir) { circuit8.screendirection = 21; } else { circuit8.screendirection = 10; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } }
	else if	(circuit8.screendirection == 21) {  if (dir) { circuit8.screendirection = 4; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 20; } }
	else if (circuit8.screendirection == 4)  {  if (dir) { circuit8.screendirection = 9; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 21; } }
	else if	(circuit8.screendirection == 9)  {  if (dir) { circuit8.screendirection = 19; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 4; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } }
	else if	(circuit8.screendirection == 19) {  if (dir) { circuit8.screendirection = 3; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 9; if (circuit8.screenidx == 0) { circuit8.screenidx = 1; } } }
	else if	(circuit8.screendirection == 3)  {  if (dir) { circuit8.screendirection = 30; } else { circuit8.screendirection = 19; if (circuit8.screenidx == 0) { circuit8.screenidx = 1; } } }
	else if (circuit8.screendirection == 30) {  if (dir) { circuit8.screendirection = 31; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 3; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } }
	else if (circuit8.screendirection == 31) {  if (dir) { circuit8.screendirection = 2; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } else { circuit8.screendirection = 30; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } }
	else if	(circuit8.screendirection == 2)  {  if (dir) { circuit8.screendirection = 0; } else { circuit8.screendirection = 31; if (circuit8.screenidx == 0 && circuit8.framecoloffset == 0) { circuit8.framecoloffset = 1; } } }
}

void WorldB::tripall1() {
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[0] += 10000.0f;
//		cRitters[i]->brain.tripping[1] += 100.0;
//		cRitters[i]->brain.tripping[2] += 100.0;
//		cRitters[i]->brain.tripping[3] += 100.0;
//		cRitters[i]->brain.tripping[4] += 100.0;
	}
}

void WorldB::tripall1(unsigned int amt) {
//	cout << "tripall1 by " << amt << endl;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[0] += (float) amt;
	}
}


void WorldB::untripall1() {
	unsigned int t = 0;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[t] -= 10000.0f;
		if (cRitters[i]->brain.tripping[t] < 0) {
			cRitters[i]->brain.tripping[t] = 0;
		}
	}
}

void WorldB::tripall2() {
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
//		cRitters[i]->brain.tripping[0] += 100.0;
		cRitters[i]->brain.tripping[1] += 10000.0f;
//		cRitters[i]->brain.tripping[2] += 100.0;
//		cRitters[i]->brain.tripping[3] += 100.0;
//		cRitters[i]->brain.tripping[4] += 100.0;
	}
}

void WorldB::tripall2(unsigned int amt) {
//	cout << "tripall2 by " << amt << endl;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[1] += (float) amt;
	}
}


void WorldB::untripall2() {
	unsigned int t = 1;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[t] -= 10000.0f;
		if (cRitters[i]->brain.tripping[t] < 0) {
			cRitters[i]->brain.tripping[t] = 0;
		}
	}
}


void WorldB::tripall3() {
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
//		cRitters[i]->brain.tripping[0] += 100.0;
//		cRitters[i]->brain.tripping[1] += 100.0;
		cRitters[i]->brain.tripping[2] += 10000.0f;
//		cRitters[i]->brain.tripping[3] += 100.0;
//		cRitters[i]->brain.tripping[4] += 100.0;
	}
}

void WorldB::tripall3(unsigned int amt) {
//	cout << "tripall3 by " << amt << endl;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[2] += (float) amt;
	}
}

void WorldB::untripall3() {
	unsigned int t = 2;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[t] -= 10000.0f;
		if (cRitters[i]->brain.tripping[t] < 0) {
			cRitters[i]->brain.tripping[t] = 0;
		}
	}
}

void WorldB::tripall4() {
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
//		cRitters[i]->brain.tripping[0] += 100.0;
//		cRitters[i]->brain.tripping[1] += 100.0;
//		cRitters[i]->brain.tripping[2] += 100.0;
		cRitters[i]->brain.tripping[3] += 10000.0f;
//		cRitters[i]->brain.tripping[4] += 100.0;
	}
}

void WorldB::tripall4(unsigned int amt) {
//	cout << "tripall4 by " << amt <<endl;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[3] += (float) amt;
	}
}

void WorldB::untripall4() {
	unsigned int t = 3;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[t] -= 10000.0f;
		if (cRitters[i]->brain.tripping[t] < 0) {
			cRitters[i]->brain.tripping[t] = 0;
		}
	}
}

void WorldB::tripall5() {
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
//		cRitters[i]->brain.tripping[0] += 100.0;
//		cRitters[i]->brain.tripping[1] += 100.0;
//		cRitters[i]->brain.tripping[2] += 100.0;
//		cRitters[i]->brain.tripping[3] += 100.0;
		cRitters[i]->brain.tripping[4] += 10000.0f;
	}
}

void WorldB::tripall5(unsigned int amt) {
//	cout << "tripall5 by " << amt <<endl;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[4] += (float) amt;
	}
}

void WorldB::untripall5() {
	unsigned int t = 4;
	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		cRitters[i]->brain.tripping[t] -= 10000.0f;
		if (cRitters[i]->brain.tripping[t] < 0) {
			cRitters[i]->brain.tripping[t] = 0;
		}
	}
}

void WorldB::follow_cv_crit() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	goto_critter_by_id(critterselection->selectedCritter->critterID);
	}
}

void WorldB::tripthis1() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[0] += 10000;
        	if (critterselection->selectedCritter->brain.tripping[0] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[0] = (float) *pill_maxtrip;
        	}
        }
}

void WorldB::untripthis1() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[0] -= 10000;
        	if (critterselection->selectedCritter->brain.tripping[0] < 0) {
        		critterselection->selectedCritter->brain.tripping[0] = 0;
        	}
        }
}


void WorldB::tripthis2() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[1] += 10000;
        	if (critterselection->selectedCritter->brain.tripping[1] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[1] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::untripthis2() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[1] -= 10000;
        	if (critterselection->selectedCritter->brain.tripping[1] < 0) {
        		critterselection->selectedCritter->brain.tripping[1] = 0;
        	}
        }
}


void WorldB::tripthis3() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[2] += 10000;
        	if (critterselection->selectedCritter->brain.tripping[2] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[2] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::untripthis3() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[2] -= 10000;
        	if (critterselection->selectedCritter->brain.tripping[2] < 0) {
        		critterselection->selectedCritter->brain.tripping[2] = 0;
        	}
        }
}


void WorldB::tripthis4() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[3] += 10000;
        	if (critterselection->selectedCritter->brain.tripping[3] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[3] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::untripthis4() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[3] -= 10000;
        	if (critterselection->selectedCritter->brain.tripping[3] < 0) {
        		critterselection->selectedCritter->brain.tripping[3] = 0;
        	}
        }
}


void WorldB::tripthis5() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[4] += 10000;
        	if (critterselection->selectedCritter->brain.tripping[4] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[4] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::untripthis5() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[4] -= 10000;
        	if (critterselection->selectedCritter->brain.tripping[4] < 0) {
        		critterselection->selectedCritter->brain.tripping[4] = 0;
        	}
        }
}


void WorldB::feedthis1() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	float feeden = 0;
        	if (critterselection->selectedCritter->energyLevel + *food_maxenergy > *critter_maxenergy) {
        		feeden = *critter_maxenergy - critterselection->selectedCritter->energyLevel;
        	} else {
        		feeden = *food_maxenergy;
        	}
        	critterselection->selectedCritter->energyLevel += feeden;
		freeEnergy -= feeden;
	}
}

void WorldB::togglelockblink1() {
        if ( critterselection->selectedCritter == 0 )
        {
//        	cout << "lockblink1 returned" << endl;
        	return;
	}
	else
        {
        	if (critterselection->selectedCritter->lockblink) {
//        		cout << "lockblink1 set to false" << endl;
        		critterselection->selectedCritter->lockblink = false;
		} else {
//			cout << "lockblink1 set to true" << endl;
			critterselection->selectedCritter->lockblink = true;
		}
	}	
}

void WorldB::togglelockblink2() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	if (critterselection->selectedCritter->lockblinkscreen) { critterselection->selectedCritter->lockblinkscreen = false; } else { critterselection->selectedCritter->lockblinkscreen = true; }
//        	cout << "lockblinkscreen set to " << critterselection->selectedCritter->lockblinkscreen << endl;
	} 
}

void WorldB::togglelockdruglevels() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	if (critterselection->selectedCritter->brain.lockdruglevels) { critterselection->selectedCritter->brain.lockdruglevels = false; } else { critterselection->selectedCritter->brain.lockdruglevels = true; }
	}	
}


void WorldB::createDirs()
{
#ifndef _WIN32
 	homedir = getenv("HOME");
 	if ( homedir.empty() ) {
//		cout << "environment variable HOME not defined/detected" << endl;
		exit(0);
	}
	progdir = homedir;	progdir.append("/.critterding");
	savedir = progdir;	savedir.append("/save");
	loaddir = progdir;	loaddir.append("/load");
	circuit8.loaddir = loaddir;	circuit8.loaddir.append("/movie");
	scrndir = progdir;	scrndir.append("/screenshots/");
#else
	char mydoc[256];
	memset(mydoc, 0, sizeof(mydoc));

	SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, mydoc);
	homedir.assign(mydoc);
	progdir = homedir;	progdir.append("\\critterding");
	savedir = progdir;	savedir.append("\\save");
	loaddir = progdir;	loaddir.append("\\load");
	circuit8.loaddir = loaddir;	circuit8.loaddir.append("\\movie");
	scrndir = progdir;	scrndir.append("\\screenshots\\");
#endif

//	cerr << progdir << endl;

	if ( !dirH.exists(progdir) ) dirH.make(progdir);
	if ( !dirH.exists(loaddir) ) dirH.make(loaddir);
	if ( !dirH.exists(circuit8.loaddir) ) dirH.make(circuit8.loaddir);
	if ( !dirH.exists(scrndir) ) dirH.make(scrndir);
}

void WorldB::resetCamera()
{
	unsigned int biggest = settings->getCVar("worldsizeX");
	if ( settings->getCVar("worldsizeY") > biggest )
		biggest = 1.4f*settings->getCVar("worldsizeY");

	if (!autocamOn || cRitters.size()==0) {
		camera.position.setOrigin( btVector3( 0.5f*settings->getCVar("worldsizeX"), 1.3f*biggest, 0.5f*settings->getCVar("worldsizeY")) );
	} else {
		btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[camcritter]->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();
		camera.position.setOrigin( btVector3(np.getX(), np.getY()+8.0f, np.getZ()) ); 
		// go to critter pos
	}
	
// 	camera.position.setRotation(btQuaternion(btVector3(1, 0, 0), btScalar(90)));
	camera.position.getBasis().setEulerZYX( -1.5707f, 0.0f, 0.0f ); // 1.5707f  (float)settings->getCVar("energy")/10

// 	camera.position = btVector3( -0.5f*settings->getCVar("worldsizeX"), -1.3f*biggest, -0.5f*settings->getCVar("worldsizeY"));
// 	camera.rotation = Vector3f( 90.0f,  0.0f, 0.0f);
}



void WorldB::makeFloor()
{
	makeDefaultFloor();
}

void WorldB::makeDefaultFloor()
{
	for ( unsigned int i=0; i < walls.size(); i++ )	
		delete walls[i];
	walls.clear();

	// Wall Constants
		float WallWidth = 0.5f;
		float WallHalfWidth = WallWidth/2.0f;
		float WallHeight = 2.0f;
		float WallHalfHeight = WallHeight/2.0f;

	// Ground Floor
		btVector3 position( settings->getCVar("worldsizeX")/2.0f, -WallHalfWidth, settings->getCVar("worldsizeY")/2.0f );
		Wall* w = new Wall( settings->getCVar("worldsizeX"), WallWidth, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
		w->color[0] = 0.30f; w->color[1] = 0.20f; w->color[2] = 0.10f;
//		w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
		walls.push_back(w);
	
	if ( settings->getCVar("worldwalls") )
	{
		// Left Wall
			position = btVector3 ( 0.0f-WallHalfWidth, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")/2.0f );
			w = new Wall( WallWidth, WallHeight, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Right Wall
			position = btVector3 ( settings->getCVar("worldsizeX")+WallHalfWidth, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")/2.0f );
			w = new Wall( WallWidth, WallHeight, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Top Wall
			position = btVector3 ( settings->getCVar("worldsizeX")/2.0f, WallHalfHeight-WallWidth, 0.0f-WallHalfWidth );
			w = new Wall( settings->getCVar("worldsizeX")+(WallWidth*2), WallHeight, WallWidth, position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Bottom Wall
			position = btVector3 ( settings->getCVar("worldsizeX")/2.0f, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")+WallHalfWidth );
			w = new Wall( settings->getCVar("worldsizeX")+(WallWidth*2), WallHeight, WallWidth, position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
	} else { // flamoot (one wall)
		// Left Wall
			position = btVector3 ( 0.0f-WallHalfWidth, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")/2.0f );
			w = new Wall( WallWidth, WallHeight, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Right Wall
			position = btVector3 ( settings->getCVar("worldsizeX")+WallHalfWidth, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")/2.0f );
			w = new Wall( WallWidth, WallHeight, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Top Wall
			position = btVector3 ( settings->getCVar("worldsizeX")/2.0f, WallHalfHeight-WallWidth, 0.0f-WallHalfWidth );
			w = new Wall( settings->getCVar("worldsizeX")+(WallWidth*2), WallHeight, WallWidth, position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
	
	}
}

// camera ops
void WorldB::camera_moveup() { camera.moveUp(0.01f); movePickedBodyFrom(); }
void WorldB::camera_movedown() { camera.moveDown(0.01f); movePickedBodyFrom(); }
void WorldB::camera_moveforward() { camera.moveForward(0.01f); movePickedBodyFrom(); }
void WorldB::camera_movebackward() { camera.moveBackward(0.01f); movePickedBodyFrom(); }
void WorldB::camera_moveleft() { camera.moveLeft(0.01f); movePickedBodyFrom(); }
void WorldB::camera_moveright() { camera.moveRight(0.01f); movePickedBodyFrom(); }

void WorldB::camera_lookup() { camera.lookUp(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_lookdown() { camera.lookDown(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_lookleft() { camera.lookLeft(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_lookright() { camera.lookRight(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_rollleft() { camera.rollLeft(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_rollright() { camera.rollRight(0.001f); calcMouseDirection(); movePickedBodyTo(); }

int WorldB::insertRandomFood(int amount, float energy)
{
	for ( int i=0; i < amount; i++ )
	{
		Food *f = new Food;
		f->energyLevel = energy;
		//f->resize();
		f->createBody( m_dynamicsWorld, findPosition() );
		
		food.push_back( f );
	}
}


void setPillColor(Pill* p) {
			if (p->molecule == 4) {            // potentiator purple or pink dunno
			        p->color[0] = 0.8f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.8f;
                                p->color[3] = 0.1f;
			} else if (p->molecule == 3) {            // excitatory agonist bright red
			        p->color[0] = 0.9f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.12f;
                                p->color[3] = 0.1f;
			} else if (p->molecule == 2) {     // excitatory antagonist dark red
			        p->color[0] = 0.3f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.03f;
                                p->color[3] = 0.1f;
			} else if (p->molecule == 1) {     // inhibitory agonist bright blue
			        p->color[0] = 0.1f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 1.0f;
                                p->color[3] = 0.1f;
			} else if (p->molecule == 0) {     // inhibitory antagonist dark blue
			        p->color[0] = 0.03f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.3f;
                                p->color[3] = 0.1f;
			}
			return;
}

int WorldB::insertNextPill(int anything)
{
		if (pill.size() < 128) {
			Pill* p  = new Pill;
			p->energyLevel = *pill_energylevel;
			p->molecule = randgen->Instance()->get(0,4);
			setPillColor(p);
			//p->energyLevel = energy;
			//		p->resize(120);
			p->createBody( m_dynamicsWorld, findPosition() );
			pill.push_back( p );
			pillidx += 1;
		}  else {
			cout << "urine alarm" << endl;
//			Pill* p = pill[pillidx];
//			p->energyLevel = *pill_energylevel;
//			p->molecule = randgen->Instance()->get(0,4);
//			setPillColor(p);
//			//pill[pillidx] = p->real_pill;
//			p->createBody( m_dynamicsWorld, findPosition() );
//			pillidx += 1;
		}
		pillidx += 1;
		if (pillidx > 128 - 1) {pillidx = 0;}
		pillsinworld += 1;
}

unsigned int WorldB::hashSpeciesColour(float r, float g, float b, float a) {
	unsigned int t = 0;
	t = int(r + (g*16.0) + (b*256.0) + (a*65536.0)) * 10;
	
	return t;
}

WorldB::~WorldB()
{
	for ( unsigned int i=0; i < food.size(); i++ )		delete food[i];
	for ( unsigned int i=0; i < pill.size(); i++ )		delete pill[i];
	for ( unsigned int i=0; i < cRitters.size(); i++ )	delete cRitters[i];
	for ( unsigned int i=0; i < walls.size(); i++ )		delete walls[i];

	free(retina);

	delete m_collisionConfiguration;
	delete m_dispatcher;
	delete m_broadphase;
	delete m_solver;
	delete m_dynamicsWorld;
	
	delete raycast;
	delete mousepicker;
}
